/**
 * Dasu Project
 *
 * File:         Main.java
 * RCS:          $Id: Main.java,v 1.15 2010/11/01 15:37:00 mas939 Exp $
 * Description:  Main class (see below)
 * Author:       Mario Sanchez, Dave Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 24, 2010 at 2:25:01 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.SocketException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.TimeZone;

import org.gudy.azureus2.core3.global.GlobalManager;
import org.gudy.azureus2.core3.logging.LogAlert;
import org.gudy.azureus2.core3.logging.Logger;
import org.gudy.azureus2.core3.util.Debug;
import org.gudy.azureus2.plugins.PluginException;
import org.gudy.azureus2.plugins.PluginListener;
import org.gudy.azureus2.plugins.UnloadablePlugin;
import org.gudy.azureus2.plugins.clientid.ClientIDManager;
import org.gudy.azureus2.plugins.config.ConfigParameter;
import org.gudy.azureus2.plugins.config.ConfigParameterListener;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadListener;
import org.gudy.azureus2.plugins.download.DownloadManager;
import org.gudy.azureus2.plugins.download.DownloadManagerListener;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.logging.LoggerChannelListener;
import org.gudy.azureus2.plugins.ui.UIInstance;
import org.gudy.azureus2.plugins.ui.UIManager;
import org.gudy.azureus2.plugins.ui.UIManagerListener;
import org.gudy.azureus2.plugins.ui.config.BooleanParameter;
import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
import org.gudy.azureus2.plugins.utils.UTTimer;
import org.gudy.azureus2.plugins.utils.Utilities;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderException;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderFactory;
import org.gudy.azureus2.pluginsimpl.local.utils.resourcedownloader.ResourceDownloaderFactoryImpl;
import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;

import edu.northwestern.dasu.MainGeneric;
import edu.northwestern.dasu.download.DownloadObserver;
import edu.northwestern.dasu.measurement.NetstatBWStats;
import edu.northwestern.dasu.measurement.NetstatBanwidthPerformer;
import edu.northwestern.dasu.stats.StatManager;
import edu.northwestern.dasu.time.SntpClient;
import edu.northwestern.dasu.ui.DasuView;
import edu.northwestern.dasu.util.AzPluginInterface;
import edu.northwestern.dasu.util.PluginInterface;
import edu.northwestern.dasu.util.Util;

import com.aelitis.azureus.core.impl.AzureusCoreImpl;
import com.azureus.plugins.aznetmon.main.RSTPacketCountPerformer;
import com.azureus.plugins.aznetmon.main.RSTPacketStats;

/**
 * @author Mario Sanchez, David Choffnes &lt;msanchez@eecs.northwestern.edu&gt;
 * 
 *         The Main class ...
 */
public class Main extends MainGeneric implements UnloadablePlugin,
		DownloadListener {

	private static UISWTInstance swtInstance = null;
	private UISWTViewEventListener myView = null;

	private static PluginInterface pi_az = null;

	private DownloadManager dm;

	private ClientIDManager cidm;
	Utilities ut;
	private BasicPluginViewModel model;
	private static LoggerChannel log;
	private ArrayList<URL> urls = new ArrayList<URL>();
	private UTTimer torrentCheckTimer;
	private UIManagerListener uiListener;
	private BasicPluginConfigModel config_model;
	private LoggerChannelListener logListener;
	private DownloadManagerListener dldListener;
	private PluginListener pluginListener;
	private PluginListener initializationCompleteListener;
	private BooleanParameter enableStatsParam;
	private BooleanParameter enableDasuParam;
	private BooleanParameter enablePopupsParam;
	private boolean initialized = false;
	

	/** downloader factory */
	private static ResourceDownloaderFactory rd_factory = ResourceDownloaderFactoryImpl
			.getSingleton();

	/** list of download managers */
	private HashMap<Download, DownloadObserver> dlObservers = new HashMap<Download, DownloadObserver>();

	private static boolean canCheckPlugins = false; // if false, plugin
													// initialization hasn't
													// finished
	/** timer for RST observations */
	protected static RSTPacketCountPerformer perf;
	private UTTimer utTimer;
	
	/** time for Netstst BW observations */
	protected static NetstatBanwidthPerformer netstatBWPerf;
	private UTTimer netstatBWutTimer;
	
	private static ArrayList<String> propertiesLocations = new ArrayList<String>();
	private static Main self;

	public Main() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.gudy.azureus2.plugins.Plugin#initialize(org.gudy.azureus2.plugins
	 * .PluginInterface)
	 */
	public void initialize(org.gudy.azureus2.plugins.PluginInterface arg0)
			throws PluginException {

		MainGeneric.startUp();
		MainGeneric.setType("Azureus");
		pi_az = new AzPluginInterface();
		((AzPluginInterface) pi_az).setPluginInterface(arg0);

		/* grab a bunch of component references */
		cidm = pi_az.getClientIDManager();
		ut = pi_az.getUtilities();

		log = pi_az.getLogger().getChannel("Dasu");

		/* register listeners for downloads */
		dldListener = new DownloadManagerListener() {

			public void downloadAdded(Download download) {
				download.addListener(Main.this);
			}

			public void downloadRemoved(Download download) {
				download.removeListener(Main.this);
			}
		};
		
		pi_az.getDownloadManager().addListener(dldListener);

		dm = pi_az.getDownloadManager();
		if (dm != null) {
			for (Download download : dm.getDownloads()) {
				if (download.getState() == Download.ST_DOWNLOADING
						|| download.getState() == Download.ST_SEEDING) {

					if (!dlObservers.containsKey(download)) {
						DownloadObserver dlObs = new DownloadObserver(download);
						dlObservers.put(download, dlObs);
					}

				} else
					download.addListener(this);
			}
		}

		/* register listeners to handle application shutdown */
		pluginListener = new PluginListener() {

			public void closedownComplete() {
			}

			public void closedownInitiated() {
				try {
					unload();
				} catch (PluginException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			public void initializationComplete() {
				// TODO Auto-generated method stub
			}
		};
		pi_az.addListener(pluginListener);

		/* For now we are getting data for Windows and OSX */// ToDo: make it run
															// on multiple OSs
		if (ut.isWindows() || ut.isOSX() || ut.isLinux()) {

			utTimer = ut.createTimer("RST Packet Monitor", false);
			perf = new RSTPacketCountPerformer(pi_az);

			perf.platform = getPlatform();
			
			netstatBWutTimer = ut.createTimer("Netstat BW Monitor", false);
			netstatBWPerf = new NetstatBanwidthPerformer(pi_az, MainGeneric.getInterfaceName());

			/* start on event within one second */
			utTimer.addEvent(1000, perf);
			netstatBWutTimer.addEvent(1000, netstatBWPerf);
			
			/* follow-up every 1 minute */
			// rstTimerEvent = utTimer.addPeriodicEvent(
			// DASUConfiguration.getInstance().getPeerUpdateIntervalSec()*1000,
			// perf);
			
		} else {
			// Currently only works with windows, Linux and MacOS
			RSTPacketStats stats = RSTPacketStats.getInstance();
			stats.setStatus("Plug-in currently only works with Windows, Linux or MacOS machines.");
			
			NetstatBWStats bwStats = NetstatBWStats.getInstance();
			bwStats.setStatus("Plug-in currently only works with Windows, Linux or MacOS machines.");
		}

		/* GUI STUFF */

		pi_az.getUtilities().getLocaleUtilities()
				.integrateLocalisedMessageBundle(
						"edu.northwestern.dasu.messages");

		UIManager ui_manager = pi_az.getUIManager();		

		// TODO add more GUI options
		config_model = ui_manager.createBasicPluginConfigModel("plugins",
				"Dasu.name");

		enableDasuParam = config_model.addBooleanParameter2(
				"dasu.config.enable.detection", "dasu.config.enable.detection",
				DasuConfiguration.getInstance().isRecordStats());
		enableDasuParam
				.addConfigParameterListener(new ConfigParameterListener() {
					public void configParameterChanged(ConfigParameter param) {
						DasuConfiguration.getInstance().setRecordStats(
								enableDasuParam.getValue());

						if (!enableDasuParam.getValue())
							DasuManager.getInstance().stop();
						else
							startDasuManager();

						MainGeneric.updateProperties();

					}
				});

		enableStatsParam = config_model.addBooleanParameter2(
				"dasu.config.enable.report", "dasu.config.enable.report",
				DasuConfiguration.getInstance().isSendToDatabase());
		enableStatsParam
				.addConfigParameterListener(new ConfigParameterListener() {
					public void configParameterChanged(ConfigParameter param) {
						DasuConfiguration.getInstance().setSendToDatabase(
								enableStatsParam.getValue());
						MainGeneric.updateProperties();
					}
				});
		
		enablePopupsParam = config_model.addBooleanParameter2(
				"dasu.config.enable.popups", "dasu.config.enable.popups",
				DasuConfiguration.getInstance().isEnablePopups());
		enablePopupsParam
				.addConfigParameterListener(new ConfigParameterListener() {
					public void configParameterChanged(ConfigParameter param) {
						DasuConfiguration.getInstance().setEnablePopups(
								enablePopupsParam.getValue());
						MainGeneric.updateProperties();
					}
				});

		config_model.addLabelParameter2("dasu.config.privacynote");
		if (pi_az.getUtilities().compareVersions("2.5.0.2",
				pi_az.getAzureusVersion()) <= 0) {
			config_model.addHyperlinkParameter2("aqualab.teamSig",
					"http://aqualab.cs.northwestern.edu");
		} else {
			config_model.addLabelParameter2("aqualab.teamSig");
		}

		model = pi_az.getUIManager().createBasicPluginViewModel("Dasu Logs");

		model.getActivity().setVisible(false);
		model.getProgress().setVisible(false);

		uiListener = new UIManagerListener() {
			public void UIAttached(UIInstance instance) {
				if (instance instanceof UISWTInstance) {
					swtInstance = (UISWTInstance) instance;
					myView = new DasuView(pi_az);
					swtInstance.addView(UISWTInstance.VIEW_MAIN,
							DasuView.VIEWID, myView);
				}
			}

			public void UIDetached(UIInstance instance) {
				if (instance instanceof UISWTInstance) {
					swtInstance.removeViews(UISWTInstance.VIEW_MAIN,DasuView.VIEWID);
					swtInstance = null;
				}
			}
		};

		pi_az.getUIManager().addUIListener(uiListener);

		// print output to log window
		logListener = new LoggerChannelListener() {
			public void messageLogged(int type, String message) {
				model.getLogArea().appendText(message + "\n");
			}

			public void messageLogged(String str, Throwable error) {
				model.getLogArea().appendText(error.toString() + "\n");
			}
		};

		log.addListener(logListener);

		/* register listeners to handle application initialization */
		initializationCompleteListener = new PluginListener() {

			public void closedownComplete() {
			}

			public void closedownInitiated() {
			}

			public void initializationComplete() {

				canCheckPlugins = true;
				getDasuTime();
				
				if (DEBUG) {
					System.out.println("\n");
					System.out.println("currentMillis(): " + new Date(System.currentTimeMillis()));
					System.out.println("currentDasu(): " + new Date(Util.currentDasuTime()));
					System.out.println("currentGMT(): " + new Date(Util.currentGMTTime()));
					System.out.println("TimeZone(): " + TimeZone.getDefault().getDisplayName(
							((TimeZone.getDefault().getDSTSavings() == 0)?false:true), TimeZone.SHORT));
					System.out.println("\n");
				}
				
				// PortManager portMgr = new PortManager(pi_az);
				// portMgr.openPort(55002);
				// AMCoordinatorV2.addTorrentDownload();

				if (DEBUG)
					System.out.println("Initializing dasu version "
							+ pi_az.getPluginVersion());

				String domain = getDomainName();
				propertiesFileName = propertiesFileName.concat("." + domain);
								
				propertiesLocations.add(pi_az.getPluginDirectoryName()
						+ java.io.File.separator + propertiesFileName);
				if (System.getProperties().getProperty("user.home") != null) {
					propertiesLocations.add(System.getProperties().getProperty(
							"user.home")
							+ java.io.File.separator + propertiesFileName);
				}

				/* load properties */
				loadProperties();

				if (DasuConfiguration.getInstance().getUUIDid() == -1)
					initializeValues();

				if (DasuConfiguration.getInstance().isRecordStats())
					startDasuManager();

				log.log("[" + new Date() + "]: Initializing Dasu version "
						+ pi_az.getPluginVersion());

				pi_az.getUtilities().createThread("Register Usage",
						new Runnable() {

							public void run() {
								StatManager.start();
							}
						});
				
				initialized = true;

			}
		};
		
		pi_az.addListener(initializationCompleteListener);


	} // end initialize

	private void loadProperties() {
		props = new Properties();

		if (DEBUG)
			System.out.println("DEBUG: Main: loadProperties()");
		
		boolean found = false;
		try {
			for (String location : propertiesLocations) {
				File properties_file = new File(location);
				// if properties file exists on its own then override any
				// properties file
				// potentially held within a jar
				if (properties_file.exists()) {
					FileInputStream fis = null;

					try {
						fis = new FileInputStream(properties_file);

						props.load(fis);
						found = true;
						break;
					} finally {
						if (fis != null) {
							fis.close();
						}
					}
				}
			}

			if (!found) {
				// properties file wasn't there, so use the jar
				ClassLoader classLoader = pi_az.getPluginClassLoader();

				if (classLoader instanceof URLClassLoader) {
					URLClassLoader current = (URLClassLoader) classLoader;

					URL url = current.findResource("dasu.properties.default");

					if (url != null) {
						props.load(url.openStream());
					} else {
						throw (new Exception(
								"failed to load dasu.properties.default from jars"));
					}
				}
			}
		} catch (Throwable e) {
			Debug.printStackTrace(e);

			String msg = "Can't read 'dasu.properties.default' for plugin '"
					+ pi_az.getPluginName() + "': file may be missing";

			Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR,
					msg));

			System.out.println(msg);
			DasuView.getInstance().disable();

			try {
				throw (new PluginException(msg, e));
			} catch (PluginException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}

	}

	/**
	 * @param downloader
	 * @param data
	 */
	protected void writeAndLoadFile(ResourceDownloader downloader,
			InputStream data) {
		File outputFile = new File(pi_az.getPluginDirectoryName()
				+ File.separatorChar
				+ downloader.getName().substring(
						downloader.getName().lastIndexOf('/')));

		if (outputFile.exists()) {
			outputFile.delete();
		}

		FileWriter out;

		try {
			out = new FileWriter(outputFile);

			int c;

			while ((c = data.read()) != -1)
				out.write(c);

			out.flush();
			out.close();

			urls.add(outputFile.toURL());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	// from here on down, these are download listener actions.
	// can be used for net positioning control by adding/removing "nearby" peers
	public void stateChanged(Download download, int old_state, int new_state) {

		if ((new_state == Download.ST_DOWNLOADING)
				|| (new_state == Download.ST_SEEDING)) {
			if (!dlObservers.containsKey(download)) {
				DownloadObserver pf = new DownloadObserver(download);
				dlObservers.put(download, pf);

			}
		} else if ((new_state == Download.ST_QUEUED)
				|| (new_state == Download.ST_STOPPED)) {
			DownloadObserver pf = dlObservers.remove(download);

			if (pf != null) {
				pf.setActive(false);
			}
		}
	}

	public void positionChanged(Download download, int oldPosition,
			int newPosition) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		unload();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.gudy.azureus2.plugins.UnloadablePlugin#unload()
	 */
	public void unload() throws PluginException {
		
		if (initialized) {
	 		MainGeneric.startShutdown();
	
			try {
				// catch all exceptions so it is sure to unload
				MainGeneric.getReporter().setExtendedConnection(true);
	
				try {
	
					dasuUnload();
					log.removeListener(logListener);
					pi_az.getUIManager().removeUIListener(uiListener);
					pi_az.getDownloadManager().removeListener(dldListener);
					pi_az.removeListener(pluginListener);
					pi_az.removeListener(initializationCompleteListener);
					swtInstance.removeViews(UISWTInstance.VIEW_MAIN,DasuView.VIEWID);
					utTimer.destroy();
					shutdownTimer.destroy();
					config_model.destroy();
					model.destroy();					
					myView = null;
	
				} catch (Exception e) {
					e.printStackTrace();
				}
	
				MainGeneric.getReporter().setExtendedConnection(false);
				MainGeneric.getReporter().disconnect();
	
				if (torrentCheckTimer != null)
					torrentCheckTimer.destroy();
	
			} catch (Exception e) {
				e.printStackTrace();
				MainGeneric.getReporter().disconnect();
			}
		}
	}

	private void startDasuManager() {
		DasuManager hm = new DasuManager();
		pi_az.getUtilities().createThread("DasuManager", hm);
	}

	/**
	 * 
	 */
	private void getDasuTime() {
		long offset = -1;
		try {
			offset = -1 * SntpClient.getOffset("pool.ntp.org");
		} catch (SocketException e) {
			//e.printStackTrace();
		} catch (UnknownHostException e) {
			//e.printStackTrace();
		} catch (IOException e) {
			//e.printStackTrace();
		}

		Util.setDasuTimeOffset(offset);

	}

	public static Properties downloadFromURL(String url, int timeout) {
		InputStream is = null;
		ResourceDownloader dl = null;

		Properties details = new Properties();
		try {
			dl = rd_factory.create(new URL(url));

			dl = rd_factory.getTimeoutDownloader(dl, timeout);

			// download the experiment list
			is = dl.download();
			details.load(is);
			dl.cancel();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ResourceDownloaderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		return details;
	}

	public static boolean isRunning() {
		return pi_az != null;
	}

	public static String getPlatform() {

		String platform = null;
		if (Main.getPluginInterface().getUtilities().isWindows())
			platform = "windows";
		else if (Main.getPluginInterface().getUtilities().isOSX())
			platform = "OSX";
		else
			platform = "linux";
		
//		if (Main.getPluginInterface().getUtilities().isLinux() ||
//				(Main.getPluginInterface().getUtilities().isFreeBSD()) ||
//				(Main.getPluginInterface().getUtilities().isUnix()))
//			platform = "linux";
		
		return platform;

	}
	
	public static PluginInterface getPluginInterface() {
		return pi_az;
	}
	
	public static GlobalManager getGlobalManager() {
		return AzureusCoreImpl.getSingleton().getGlobalManager();
	}

	public static LoggerChannel getLogger() {
		if (log == null) {

		}
		return log;
	}

	protected static ArrayList<String> getPropertiesLocations() {
		return propertiesLocations;
	}

	public static boolean isDoneInitializing() {
		return canCheckPlugins;
	}

}
