/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The MainGeneric class ...
 */
/**
 * Dasu Project
 *
 * File:         MainGeneric.java
 * RCS:          $Id: MainGeneric.java,v 1.13 2010/11/01 15:37:00 mas939 Exp $
 * Description:  MainGeneric class (see below)
 * Author:       Mario Sanchez, Dave Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 24, 2010 at 2:26:06 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import java.util.Random;

import org.gudy.azureus2.plugins.download.Download;
import org.xbill.DNS.DClass;
import org.xbill.DNS.ExtendedFlags;
import org.xbill.DNS.Message;
import org.xbill.DNS.Name;
import org.xbill.DNS.Record;
import org.xbill.DNS.ReverseMap;
import org.xbill.DNS.Section;
import org.xbill.DNS.SimpleResolver;
import org.xbill.DNS.TSIG;
import org.xbill.DNS.TextParseException;
import org.xbill.DNS.Type;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.asn.NetworkAdminASNLookupImpl;
import edu.northwestern.dasu.database.DatabaseReporter;
import edu.northwestern.dasu.database.NullReporter;
import edu.northwestern.dasu.database.PhpReporter;
import edu.northwestern.dasu.stats.StatManager;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.timer.AzureusTimer;
import edu.northwestern.dasu.timer.ITimer;
import edu.northwestern.dasu.timer.ITimerEvent;
import edu.northwestern.dasu.timer.ITimerEventPerformer;
import edu.northwestern.dasu.util.DasuPluginInterface;
import edu.northwestern.dasu.util.PingManager;
import edu.northwestern.dasu.util.PluginInterface;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez, Dave Choffnes &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The MainGeneric class ...
 */
public class MainGeneric {
	
	    protected static Properties props;

	    /** time in seconds before azureus stops reporting stats */
		protected static final boolean DEBUG = false;
	    public static boolean RUN_EXPERIMENT_MANAGER = true;

	    private static Integer MAX_PINGS = 5;
		private static Integer MAX_TR = 10;
		private static Integer MAX_ABW = 2;
		private static Integer PROBE_INTERVAL = 20; //mins
		
	    protected static PluginInterface pi = new DasuPluginInterface();
		private static String type = "NotAzurues";
		private static String publicIp;
		private static String dasuDomain;
		private static boolean isShuttingDown = false;
		protected static ITimer shutdownTimer;
		private static Random rand;
		protected static String propertiesFileName = "dasu.properties";
		
		public static Integer getMaxNumProbes( String probeType ) {

			if (probeType.equals("ping"))
				return MAX_PINGS;
			else if (probeType.equals("tr"))
				return MAX_TR;
			else 
				return MAX_ABW;
		}

		public static Integer getProbeInterval() {
			return PROBE_INTERVAL;
		}

		
		public static String getPublicIpAddressFirstTime() {
			if (isAzureus()){
				try {
				if (Main.getPluginInterface().getUtilities().getPublicAddress() != null) {
		            NetworkAdminASNLookupImpl.getInstance().setAddress(
		            		Main.getPluginInterface().getUtilities().getPublicAddress());
					String ip = Main.getPluginInterface().getUtilities().getPublicAddress().toString().substring(1);
					
					publicIp = ip;
		            return ip;
		        } else {
		        	String s = getLocalPublicAddress();
					if (s.startsWith("192.168")) return null;
					publicIp = s;
					return s;
		        }
				} catch (Exception e){
					System.err.println(e.getMessage());
					return null;
				}
			}
			return null;
		}
		
		public static String getPublicIpAddress() {
			if (isAzureus()){
				try {
				if (Main.getPluginInterface().getUtilities().getPublicAddress() != null) {
		            NetworkAdminASNLookupImpl.getInstance().setAddress(
		            		Main.getPluginInterface().getUtilities().getPublicAddress());
					String ip = Main.getPluginInterface().getUtilities().getPublicAddress().toString().substring(1);
					
	            	if (DasuConfiguration.getInstance().getUUIDid() == -1)
	            		initializeValues();

		            if (publicIp!=null && !ip.equals(publicIp)){
						Statistics.getInstance().reportIpChange(publicIp, ip, getLocalAddress(ip), Util.currentGMTTime());
					} else if (publicIp==null && ip!=null && ip.length()>0){
						Statistics.getInstance().reportIpChange(ip, ip, getLocalAddress(ip), Util.currentGMTTime());
					}
					publicIp = ip;
		            return ip;
		        } else {
		        	String s = getLocalPublicAddress();
					if (s.startsWith("192.168")) return null;
					publicIp = s;
					return s;
		        }
				} catch (Exception e){
					return null;
				}
			}
			return null;
		}

		private static String getLocalAddress(String ip) throws UnknownHostException {
			InetAddress addresses[] = InetAddress.getAllByName(InetAddress.getLocalHost().getHostName());
			// TODO verify that this always picks the "right" IP address
			//s = addresses[addresses.length-1].getHostAddress();
			if (ip!=null){
				for (InetAddress addr : addresses){
					if (addr.getHostAddress().equals(ip)) return addr.getHostAddress();
				}
			}
			
			for (InetAddress addr : addresses){
				if (addr.isSiteLocalAddress()) return addr.getHostAddress();
			}
			return "";
		}
		
		/* return eth0 by default if the interface cant' be found */
		public static String getInterfaceName() {
			
			String ret = "eth0";
			try {
				NetworkInterface eth = NetworkInterface.getByInetAddress(InetAddress.getLocalHost());
				if (eth != null)
					ret = eth.getDisplayName();
				
			} catch (UnknownHostException e) {
				return "eth0";
			} catch (SocketException e) {
				return "eth0";
			}
			
			return ret;

		}

		private static String getLocalPublicAddress() throws UnknownHostException {
			InetAddress addresses[] = InetAddress.getAllByName(InetAddress.getLocalHost().getHostName());
			// TODO verify that this always picks the "right" IP address
			//s = addresses[addresses.length-1].getHostAddress();
			for (InetAddress addr : addresses){
				if (!addr.isSiteLocalAddress()) return addr.getHostAddress();
			}
			return "";
		}
		
		public static boolean isAzureus(){
			return type.equals("Azureus");
		}

		public static void setType(String string) {
			type = string;
		}
				
	    protected static void initializeValues() {
	    	DasuConfiguration hc = DasuConfiguration.getInstance();
	    	hc.loadProperties(props);
	    }
	
		public static boolean isShuttingDown(){
			return isShuttingDown;
		}
		
		protected static void startUp(){
			isShuttingDown = false;
			PingManager.activate();
		}

		public static Random getRandom() {
			if (rand==null){
				rand = new Random();
			}
			return rand;
		}

		public static void createThread(String name, Runnable target) {
			if (isAzureus()) 
				Main.getPluginInterface().getUtilities().createThread(name, target);
		}

		public static DatabaseReporter getReporter() {
			// TODO Auto-generated method stub
			if (!DasuConfiguration.getInstance().isRecordStats()) 
				return NullReporter.getInstance();
			
			if (isAzureus()) 
				return PhpReporter.getInstance();
			else return NullReporter.getInstance();
		}
		
		public static ITimer createTimer(String name) {
			if (isAzureus()) {
				return new AzureusTimer(Main.getPluginInterface().getUtilities().createTimer(name));

			
			}
			else return null;
		}
		
		 protected void dasuUnload(){
		      			 
	            if (DEBUG) System.out.println("Stopping ping manager");
		    	PingManager.stop();
		    	
		        if (DEBUG) System.out.println("Stopping dasu manager");
		        DasuManager.getInstance().stop();

		    	if (DEBUG) System.out.println("Flushing stat manager");
		    	StatManager.flush();
		    	try {
		    		if (DEBUG) System.out.println("Stopping stat manager");
		    		StatManager.stop();
		    	} catch (Exception e){
		    		e.printStackTrace();
		    	}
		    	if (DEBUG) System.out.println("Flushing statistics");
	            Statistics.getInstance().flush();
		        
		        if (DEBUG) System.out.println("Stopping statistics");
		        Statistics.getInstance().stop();

		        updateProperties();
		}
		 
	    public static PluginInterface getPluginInterface() {
	        // TODO Auto-generated method stub
	        if (isAzureus()) return Main.getPluginInterface();
	    	return pi;
	    }
	    
		protected static void startShutdown(){
			shutdownTimer = createTimer("DasuShutdownTimer");
			shutdownTimer.addEvent(System.currentTimeMillis()+DasuConfiguration.getInstance().getReportingTimeout()*1000, 
					new ITimerEventPerformer(){

						public void perform(ITimerEvent event) {
							isShuttingDown = true;
							MainGeneric.getReporter().stopReporting();
							
						}}
			);
		}

		protected static ArrayList<String> getPropertiesLocations() {
			if (isAzureus()) 
				return Main.getPropertiesLocations();

			ArrayList<String> locs = new ArrayList<String>();
			locs.add(getPluginInterface().getPluginDirectoryName() +
						java.io.File.separator + propertiesFileName);
			return locs;
		}
		
		public static String getDomainName() {
					
			if ((dasuDomain != null) && (!dasuDomain.equals("default")))
				return dasuDomain;
			
			String myDomain = "default";
			String myIp = null;

			try {
				
				myIp = Main.getPublicIpAddressFirstTime();
				
				if (DEBUG)
					System.out.println("getPublicIpAddressFirstTime() => ip: " + myIp);
				

				if (myIp != null) {
					Record[] records = doLookup(new String[]{"-x", myIp});
					if (records == null)
						return myDomain;
					
					String domain = records[0].toString();
				
					if (domain != null) {
					
						String[] parts = domain.split("\\.");
						int length = parts.length;
						
						/* 
						 * If last part of domain name is 2 character's long, 
						 * then it contains CC top level domain
						 */
						if ((length > 3) && ( parts[length-1].length() < 3 ))
							myDomain = parts[length-3] + "." +
								parts[length-2] + "." + parts[length-1];
							
						else 
							myDomain = parts[length-2] + "." + parts[length-1];
						
					}
				}
											
				
			} catch (IOException e) {
				e.printStackTrace();
			}
			
			if (DEBUG) {
				System.out.println("!!!!!!" + myIp + "!!!!!");
				System.out.println("!!!!!!" + myDomain + "!!!!!");
			}
			
			dasuDomain = myDomain;
			return myDomain;
			
		}
		
		public static boolean isValidIp(final String ip) {
	        return ip.matches("^[\\d]{1,3}\\.[\\d]{1,3}\\.[\\d]{1,3}\\.[\\d]{1,3}$");
	    }
		
	    public static String getDatabaseName(){
	    	if (isAzureus()) 
	    		return "dasu";

	    	return "null";
	    }
	    
	    public static String getDatabaseHost(){
	    	if (isAzureus()) return "merlot.cs.northwestern.edu";
	    	else return "janus.cs.northwestern.edu";
	    }

		public static Properties downloadFromURL(String url, int timeout) {
			if (isAzureus()) 
				return Main.downloadFromURL( url, timeout);

			else 
				return new Properties();
		}
		
		public static boolean downloadFileFromURL(String url, String localFileName, String destinationDir) {

			int chunkSize=1024;
			URLConnection  uCon = null;
			OutputStream outStream = null;
			InputStream is = null;
			
			try {
				URL Url;
				byte[] buf;
				int ByteRead,ByteWritten=0;
				Url= new URL(url);
				outStream = new BufferedOutputStream(new
				FileOutputStream(destinationDir+"/"+localFileName));

				uCon = Url.openConnection();
				
				is = uCon.getInputStream();
				buf = new byte[chunkSize];
				while ((ByteRead = is.read(buf)) != -1) {
					outStream.write(buf, 0, ByteRead);
					ByteWritten += ByteRead;
				}

				if (DEBUG)
					System.out.println("Downloaded Successfully (" + url + ")");
				
				
			} catch (Exception e) {
				System.out.println("downloadFileFromURL: Timeout: " + url);
				return false;
				
			} finally {
				try {
					
					if (is != null)
						is.close();
					outStream.close();
					
				} catch (IOException e) {
					if (DEBUG)
						System.out.println("downloadFileFromURL: File not found!: " + localFileName);
					return false;
				}
			}
			return true;

		}
		
	    public static void updateProperties() {
	    	DasuConfiguration hc = DasuConfiguration.getInstance();
	    	hc.writeProperties(props, getPropertiesLocations());
	    }

			
	    
		/**
		 * Does a remote lookup. If the -x option is specified, only authoritative
		 * name servers are returned. Otherwise, an array of resolved IP addresses
		 * is returned
		 * 
		 * @param argv
		 *            the dig command parameterse
		 * @return
		 * @throws IOException
		 */
		public static Record[] doLookup(String[] argv) throws IOException {

			org.xbill.DNS.Name name = null;
			int type = org.xbill.DNS.Type.A;
			int dclass = DClass.IN;
			String server = null;
			int arg;
			Message query;
			Message response = null;
			Record rec;
			SimpleResolver res = null;
			boolean printQuery = false;

			if (argv.length < 1) {
				throw new RuntimeException("Invalid dig format!");
			}

			try {
				arg = 0;

				if (argv[arg].startsWith("@")) {
					server = argv[arg++].substring(1);
					res = new SimpleResolver(server);
				}

				String nameString = argv[arg++];

				if (nameString.equals("-x")) {
					name = ReverseMap.fromAddress(argv[arg++]);
					type = Type.PTR;
					dclass = DClass.IN;
				} else {
					name = Name.fromString(nameString, Name.root);
					type = Type.value(argv[arg]);

					if (type < 0) {
						type = Type.A;
					} else {
						arg++;
					}

					dclass = DClass.value(argv[arg]);

					if (dclass < 0) {
						dclass = DClass.IN;
					} else {
						arg++;
					}
				}

				while (argv[arg].startsWith("-") && (argv[arg].length() > 1)) {
					switch (argv[arg].charAt(1)) {
					case 'p':

						String portStr;
						int port;

						if (argv[arg].length() > 2) {
							portStr = argv[arg].substring(2);
						} else {
							portStr = argv[++arg];
						}

						port = Integer.parseInt(portStr);

						if ((port < 0) || (port > 65536)) {
							System.out.println("Invalid port");

							return null;
						}

						res.setPort(port);

						break;

					case 'b':

						String addrStr;

						if (argv[arg].length() > 2) {
							addrStr = argv[arg].substring(2);
						} else {
							addrStr = argv[++arg];
						}

						InetAddress addr;

						try {
							addr = InetAddress.getByName(addrStr);
						} catch (Exception e) {
							System.out.println("Invalid address");

							return null;
						}

						res.setLocalAddress(addr);

						break;

					case 'k':

						String key;

						if (argv[arg].length() > 2) {
							key = argv[arg].substring(2);
						} else {
							key = argv[++arg];
						}

						res.setTSIGKey(TSIG.fromString(key));

						break;

					case 't':
						res.setTCP(true);

						break;

					case 'i':
						res.setIgnoreTruncation(true);

						break;

					case 'e':

						String ednsStr;
						int edns;

						if (argv[arg].length() > 2) {
							ednsStr = argv[arg].substring(2);
						} else {
							ednsStr = argv[++arg];
						}

						edns = Integer.parseInt(ednsStr);

						if ((edns < 0) || (edns > 1)) {
							System.out.println("Unsupported " + "EDNS level: "
									+ edns);

							return null;
						}

						res.setEDNS(edns);

						break;

					case 'd':
						res.setEDNS(0, 0, ExtendedFlags.DO, null);

						break;

					case 'q':
						printQuery = true;

						break;

					default:
						System.out.print("Invalid option: ");
						System.out.println(argv[arg]);
					}

					arg++;
				}
			} catch (ArrayIndexOutOfBoundsException e) {
				if (name == null) {
					// log.log("Name is null!");
					throw new RuntimeException("Name is null!");
				}
			} catch (UnknownHostException e) {
				if (DEBUG)
					System.out.println(e);
			} catch (TextParseException e) {
				if (DEBUG)
					System.out.println(e);
			}

			if (res == null) {
				try {
					res = new SimpleResolver();
				} catch (UnknownHostException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					System.out.println(e);
				}
			}

			if (name == null) {
				return null;
			}

			rec = Record.newRecord(name, type, dclass);
			query = Message.newQuery(rec);

			if (printQuery) {
				System.out.println(query);
			}


			response = res.send(query);

			// return only the authority section
			if ((response != null) && argv[0].equals("-x")) {
				if (argv.length == 2)
					return response.getSectionArray(Section.ANSWER);
			}

			// return the actual ip addresses
			if (response != null) {
				return response.getSectionArray(Section.ANSWER);
			}

			return null;
		}
		

		public static HashMap<String, String> getPropertiesFromServer(String url) {

			Properties listProperties = null;
			HashMap<String, String> retList = new HashMap<String, String>();

			/* Obtain list of 'objects' to perform specified probe */
			while (listProperties == null || listProperties.isEmpty()) {

				if (DEBUG)
					System.out.println("DasuManager: Connecting to " + url);

				listProperties = MainGeneric.downloadFromURL(url, 30 * 1000);
				if (listProperties.isEmpty()) {
					if (DEBUG) {
						System.out.println("DasuManager: ERROR loading " + url);
						System.out
								.println("DasuManager: Retrying url in 60 seconds... ");
					}
					try {
						Thread.sleep(60 * 1000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}

			if (listProperties != null) {

				for (Integer i = 1; listProperties.containsKey("key."
						+ i.toString()); i++) {
					String key = "key." + i.toString();
					String value = (String) listProperties.get(key);
					retList.put(key, value);
				}
			}

			return retList;
		}
		
		
		
		/*SERIALIZABLE OPERATIONS API*/
		
	    public static void saveObject(Serializable object, String filename) {
	    	ObjectOutputStream objstream;
			try {

				objstream = new ObjectOutputStream(new FileOutputStream(filename));
		        objstream.writeObject(object);
		        objstream.close();

			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	    }

	    /**
	     * Loads an object.
	     */
	    public static Object loadObject(String filename) {

	    	ObjectInputStream objstream;
	        Object object = null; 
	        
			try {
				
		        File f = new File(filename);
		        if (f.isFile()) {
					objstream = new ObjectInputStream(new FileInputStream(f));
			        object = objstream.readObject();
			        objstream.close();
		        }

			} catch (FileNotFoundException e) {
				//e.printStackTrace();
			} catch (IOException e) {
				//e.printStackTrace();
				//when versionID for persistenData() class changed, then don't load,
				//ignore error and simply override file
			} catch (ClassNotFoundException e) {
				//e.printStackTrace();
			}
 			
	        return object;
	    }
			
		public static int getId(Download d) {
			
			MessageDigest md;
			try {
				md = MessageDigest.getInstance ("SHA");
			
			
		        md.update(d.getName().getBytes());
		        md.update(Util.convertLong(d.getCreationTime()));
		        byte hash[] = md.digest();
		        String s = new String(hash);
		        return s.hashCode();
			} catch (NoSuchAlgorithmException e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
			}
			return (int)(d.getName().hashCode() + d.getCreationTime());
			
		}

	    
}
