/**
 * Dasu Project
 *
 * File:         PersistentData.java
 * RCS:          $Id: PersistentData.java,v 1.8 2010/11/01 15:37:00 mas939 Exp $
 * Description:  PersistentData class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 24, 2010 at 2:27:04 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.LinkedHashMap;

import edu.northwestern.dasu.measurement.ProbeType;
import edu.northwestern.dasu.measurement.SignalSampleV2;
import edu.northwestern.dasu.measurement.TestType;
import edu.northwestern.dasu.timer.CronjobEntry;
import edu.northwestern.dasu.ui.UIManager;
import edu.northwestern.dasu.util.HashMapCache;
import edu.northwestern.dasu.util.Pair;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The PersistentData class ...
 */
public class PersistentData implements Serializable{

	private static final long serialVersionUID = 1221642255540144448L;

	private int maxLogEntries = 
	DasuConfiguration.getInstance().getMaxLogEntries();

	private Hashtable<String, CronjobEntry> cronjobSchedule = 
		new Hashtable<String, CronjobEntry>();

	private HashMapCache<Integer, Long> taskLogHistory =
	new HashMapCache<Integer, Long>(maxLogEntries);

	/*holds probe/test history for the past minute*/
	private LinkedHashMap<Pair<ProbeType, String>, Long> shortTermProbeHistory = 
		new LinkedHashMap<Pair<ProbeType, String>, Long>();
	private LinkedHashMap<Pair<TestType, String>, Long> shortTermTestHistory = 
		new LinkedHashMap<Pair<TestType, String>, Long>();
	private Hashtable<ProbeType, Integer> shortTermProbeTally = 
		new Hashtable<ProbeType, Integer>();
	private Hashtable<TestType, Integer> shortTermTestTally = 
		new Hashtable<TestType, Integer>();
	
	/*holds the hourly tally (count) of probes*/
	private Hashtable<ProbeType, Integer> hourlyProbeTally = 
		new Hashtable<ProbeType, Integer>();
	private Hashtable<TestType, Integer> hourlyTestTally = 
		new Hashtable<TestType, Integer>();

	/* holds historic data */
	private Hashtable<String, SignalSampleV2> historicSamplesV2 =
		new Hashtable<String, SignalSampleV2>();
	
	private Pair<String, Long> ispInfo = new Pair<String, Long>(null, null);
	
	/* message log */
	private LinkedHashMap<String, Long> messageLog = new LinkedHashMap<String, Long>();
	
	private UIManager uiManager = new UIManager();
	
	public PersistentData() {
		
	}
	
	public synchronized Hashtable<String, CronjobEntry> getCronjobSchedule() {
		return this.cronjobSchedule;
	}
	
	public synchronized HashMapCache<Integer, Long> getTaskLogHistory() {
		return taskLogHistory;
	}

	public synchronized LinkedHashMap<Pair<ProbeType, String>, Long> getShortTermProbeHistory() {
		return shortTermProbeHistory;
	}

	public synchronized LinkedHashMap<Pair<TestType, String>, Long> getShortTermTestHistory() {
		return shortTermTestHistory;
	}
	
	public synchronized LinkedHashMap<String, Long> getMessageLog() {
		return messageLog;
	}

	public synchronized Hashtable<ProbeType, Integer> getShortTermProbeTally() {
		return shortTermProbeTally;
	}

	public synchronized Hashtable<TestType, Integer> getShortTermTestTally() {
		return shortTermTestTally;
	}

	public synchronized Hashtable<ProbeType, Integer> getHourlyProbeTally() {
		return hourlyProbeTally;
	}

	public synchronized Hashtable<TestType, Integer> getHourlyTestTally() {
		return hourlyTestTally;
	}

	public synchronized Hashtable<String, SignalSampleV2> getHistoricSamplesV2() {
		return historicSamplesV2;
	}
	
	public void setCronjobSchedule(Hashtable<String, CronjobEntry> sched) {
		this.cronjobSchedule = sched;
	}

	public void setTaskLogHistory(HashMapCache<Integer, Long> taskLogHistory) {
		this.taskLogHistory = taskLogHistory;
	}

	public void setShortTermProbeHistory(
			LinkedHashMap<Pair<ProbeType, String>, Long> shortTermProbeHistory) {
		this.shortTermProbeHistory = shortTermProbeHistory;
	}

	public void setShortTermTestHistory(
			LinkedHashMap<Pair<TestType, String>, Long> shortTermTestHistory) {
		this.shortTermTestHistory = shortTermTestHistory;
	}

	public void setShortTermProbeTally(
			Hashtable<ProbeType, Integer> shortTermProbeTally) {
		this.shortTermProbeTally = shortTermProbeTally;
	}

	public void setShortTermTestTally(
			Hashtable<TestType, Integer> shortTermTestTally) {
		this.shortTermTestTally = shortTermTestTally;
	}

	public void setHourlyProbeTally(
			Hashtable<ProbeType, Integer> hourlyProbeTally) {
		this.hourlyProbeTally = hourlyProbeTally;
	}

	public void setHourlyTestTally(
			Hashtable<TestType, Integer> hourlyTestTally) {
		this.hourlyTestTally = hourlyTestTally;
	}
	
	public Pair<String, Long> getIspInfo() {
		return ispInfo;
	}

	public void setIspInfo(Pair<String, Long> ispInfo) {
		this.ispInfo = ispInfo;
	}
	
	public void setMessageLog(LinkedHashMap<String, Long> log) {
		this.messageLog = log;
	}


	public UIManager getUiManager() {
		return uiManager;
	}

	public void setUiManager(UIManager uiManager) {
		this.uiManager = uiManager;
	}

	
}