package edu.northwestern.dasu.database;

import java.io.IOException;
import java.nio.charset.Charset;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import edu.northwestern.dasu.MainGeneric;
import edu.northwestern.dasu.measurement.DasuTraceRouteModuleV2.TraceEntry;


public interface DatabaseReporter {

	public abstract void setExtendedConnection(boolean extend);

	/**
	 * Establishes a connection to the databse.
	 * 
	 * @return false if the connection was not made; true otherwise
	 */
	public abstract boolean connect();

	/**
	 * @param config
	 * @throws ClassNotFoundException
	 * @throws IOException
	 * @throws SQLException
	 */
	public abstract int doInsert(String insert, String select)
			throws IOException, ClassNotFoundException, SQLException;

	public abstract void doCreateTable(String s) throws SQLException,
			IOException, ClassNotFoundException;

	public abstract boolean canConnect();

	/**
	 * @return
	 */
	public abstract int getNextPosition();

	/**
	 * @param customerId
	 * @param edgeId
	 * @param peerId
	 * @param i
	 * @param time
	 * @param timeZoneId
	 */
	public abstract boolean insertEvent(int customerId, int edgeId, int peerId,
			int eventType, long time, int timeZoneId);

	/**
	 * @param edgeServer
	 * @return
	 * @throws SQLException
	 */
	public abstract int makeId(String tableName, String colName, String value);

	/**
	 * @param customerId
	 * @param peerId
	 * @param edge1Id
	 * @param edge2Id
	 * @param time
	 */
	public abstract boolean insertDigResult(int customerId, int peerId,
			int edge1Id, int edge2Id, long time);

	/**
	 * 
	 */
	public abstract void disconnect();

	/**
	 * @param edgeId
	 * @param myId
	 * @param destId
	 * @param middleId
	 * @param rtt
	 * @param time
	 */
	public abstract boolean insertPingResultDasu(int destId, int sourceId,float rtt,  
			long time, int destType, int taskId);
	
	public abstract boolean insertHttpResultDasu(int sourceId, int url,  
    		Double downloadDuration, Double downloadSpeed, long time, 
    		int noBytes, int taskId);

	public abstract boolean insertSpeedTestResult(int sourceId, long testTime, 
			int downloadSpeed, int uploadSpeed, int encryption, char type, 
			int downloadSpeedNetstat, int uploadSpeedNetstat);
	
	public abstract boolean insertUpnpTestResult(int sourceId, long testTime, 
        	int port, String protocol, int success);
	
	public boolean insertNdtResultDasu(int myId, String osName, String osArch,
			String osVer, String interfaceName, String javaVer, String server,
			double rate, double rate2, double mylink, int mismatch,
			double avgrtt, int packetSize, 
			int connectionStalledTimes,
			double connectionIdlePercentage, int packetQueuing,
			int packetQueuing2,
			double throughputPacketQueueingDetectedPercentage,
			double connectionReceiverLimitedPercentage,
			double connectionSenderLimitedPercentage,
			double connectionNetworkLimitedPercentage, int RFC2018,
			int RFC896, int RFC3168, int RFC1323TimeStamping,
			int RFC1323WindowScaling, int serverScalingFactor,
			int clientScalingFactor, int serverBehindFirewall,
			int clientBehindFirewall, double theoreticalNetworkLimit,
			double serverBuffer, double serverThroughputLimit,
			double clientBuffer, double clientThroughputLimit, long time,
			int failed, String errorMsg, double c2sRateNetstat, double s2cRateNetstat,
			double networkFlowControlThroughputLimit, int congestion, int congestionSignals,
			int fastRetran, int timeouts);

	
	public abstract boolean insertRSTStat(int sourceId, int destId, 
			long downloadId, int numRst, long timeRecorded, int type,
			int port, String bitTorrentClient, String source, int isSeed, 
			int amSeed, int numSeeds, int numPeers, String encryptionLevel);
	
	public abstract boolean insertRemotePeerConnectionStat(int sourceId, int destId, 
			long timeRecorded, int type,
			int port, String bitTorrentClient, int isSeed, 
			int amSeed, int numSeeds, int numPeers, String encryptionLevel);
	
	public abstract boolean insertDataTransferResult(long time,
			int experimentId, byte type, int size, int sourceId, int destId,
			int currentId, double raceResultPrimary, double raceResultSecondary);

	/**
	 * returns the id for the entry description
	 * 
	 * @param myId
	 * @param customerId
	 * @return
	 */

	public abstract void reportError(Exception e);

	public abstract long insertTraceEntry(int sourceId, int destId, long time, 
			int entries, int taskId);

	public abstract void insertTraceEntryData(long entryId, int[] myRouterIds,
			TraceEntry te);
	
	public abstract long insertScrapeEntry(String type, long peerId, String url,
			String hash, int entries, int numSeeds, int numLeechers, int taskId, long time);
	
	public abstract void insertScrapeEntryData(long entryId,
			long peerId, int tcp, int udp);


	/**
	 * Inserts local peer stats.
	 * 
	 * @param selfId
	 *            local peer id
	 * @param currentTime
	 *            the time of the stat being reported
	 * @param uploadRate
	 *            the upload rate at that time
	 * @param downloadRate
	 *            the download rate at that time
	 * @param b 
	 * @param j 
	 * @param i 
	 * @return
	 */
	public abstract boolean insertLocalPeerStatDynamic(int selfId,
			long currentTime, int uploadRate, int downloadRate, 
			int maxDownloadRate, int maxUploadRate, boolean isAutoSpeed, int rst, 
			int numConnections, int passiveOpens, int activeOpens, int failedConns,
			int numTorrents, int numSeedingTorrents, int numLeechingTorents, 
			int numConnectedSeeds, int numConnectedLeechers,
			long bytesSentLpsd, long bytesReceivedLpsd, long bytesSentNetstat, long bytesReceivedNetstat, long swarmPeerSpeed);

	/**
	 * Inserts once-per-session peer stats
	 * 
	 * @param selfId
	 *            the peer id
	 * @param uptime
	 *            the peer session lifetime
	 * @param maxUploadRate
	 *            the max upload rate
	 * @param maxDownloadRate
	 *            the max download rate
	 */

	public abstract void insertLocalPeerStatStatic(int selfId, long uptime,
			int maxUploadRate, int maxDownloadRate, int maxPeerConnectionsGlobal, 
			int maxPeerConnectionsPerTorrent, int maxPeerConnectionsPerTorrentSeeding, 
			int maxUploadRateSeeding, int maxSeedsPerTorrent, int lazyBitfield, int requireEncryption,
			String encryptionMinLevel, int allowNonEncryptedOUT, int allowNonEncryptedIn,
			int useCryptoPortExtension, int tcpPortIn, int udpPortIn);

	/**
	 * Reports once-per-session peer stats.
	 * 
	 * @param selfId
	 *            the id of the local peer
	 * @param peerId
	 *            the id of the peer being reported on
	 * @param peerClient
	 *            the id of the client software being used
	 * @param uptime
	 *            the duration of the connection
	 * @param tcpPort
	 *            the tcp port being used
	 * @param udpPort
	 *            the udp port being used
	 * @param sameCluster
	 *            true if node is in same cluster
	 */
	public abstract void insertRemotePeerStatStatic(int selfId, int peerId,
			int peerClient, long uptime, int tcpPort, int udpPort,
			boolean sameCluster, int type, String encryption, long pieceSize);

	/**
	 * Reports periodic data from remote peers.
	 * 
	 * @param selfId
	 *            local peer
	 * @param peerId
	 *            remote peer (being measured)
	 * @param currentTime
	 *            time of measurement
	 * @param bytesReceivedFromPeer
	 *            bytes received at that point
	 * @param bytesSentToPeer
	 *            bytes sent at that point
	 * @param currentReceiveRate
	 *            current download rate
	 * @param currentSendRate
	 *            current upload rate
	 * @param isSeed
	 *            whether it is a seed
	 * @param ping
	 *            ping latency
	 * @param isInteresting
	 * @param isInterested
	 * @return
	 */
	public abstract boolean insertRemotePeerStatDynamic(int selfId, int peerId,
			long currentTime, int bytesReceivedFromPeer, int bytesSentToPeer,
			int currentReceiveRate, int currentSendRate, boolean isSeed,
			double ping, boolean isInterested, boolean isInteresting);

	/**
	 * 
	 * @param selfId
	 *            peer id
	 * @param downloadId
	 *            the id for this download for this session
	 * @param currentTime
	 *            current time in ms
	 * @param downloadingTime
	 *            time spent downloading (s)
	 * @param seedingTime
	 *            time spent seeding (s)
	 * @param maxDownloadRateLimit
	 *            dl rate limit
	 * @param maxUploadLimit
	 *            upload rate limit
	 * @param totalUploaded
	 *            total bytes uploaded for this download
	 * @param totalDownloaded
	 *            total bytes downloaded for this download
	 * @param size
	 *            size of the download
	 * @param pieceSize
	 *            piece size for dl
	 */
	public abstract void insertDownloadStatSummary(int selfId, int downloadId,
			long currentTime, long downloadingTime, long seedingTime,
			int maxDownloadRateLimit, int maxUploadLimit, long totalUploaded,
			long totalDownloaded, long size, long pieceSize);

	/**
	 * 
	 * @param selfId
	 *            peer id
	 * @param downloadId
	 *            download id
	 * @param timeRecorded
	 *            time recorded
	 * @param numPeers
	 *            num peers connected
	 * @param availability
	 *            data availability
	 * @param completed
	 *            completion in thousands
	 * @param discarded
	 *            num bytes discarded
	 * @param downloadRate
	 *            download rate
	 * @param uploadRate
	 *            upload rate
	 * @param shareRatio
	 *            share ratio in thousandths
	 * @param numKnownNonSeeds
	 *            total non seeds known about
	 * @param numKnownSeeds
	 *            total seeds known about
	 * @param state
	 *            download state
	 * @param priority
	 * @param uploaded 
	 * @param downloaded 
	 * @param position 
	 * @param numOnoPeers 
	 * @param totalNumKnownSeeds 
	 * @param totalNumKnownNonSeeds 
	 * @param hashFails 
	 * @return success
	 */
	public abstract boolean insertDownloadStatDynamic(int selfId,
			int downloadId, long timeRecorded, int numPeers,
			float availability, int completed, int discarded, int downloadRate,
			int uploadRate, int shareRatio, int numKnownNonSeeds,
			int numKnownSeeds, int state, int priority, long downloaded, 
			long uploaded, int position,
			int totalNumKnownNonSeeds, int totalNumKnownSeeds, long hashFails,
			int connectedSeeds, int connectedLeechers, long swarmPeerSpeed);

	public abstract void registerUsage(int myId, long GMTTime, String uuid, long currentTime,
			String timeZone, boolean failed);

	/**
	 * Gets the update interval, in seconds
	 * 
	 * @return
	 */
	public abstract int getRecordingInterval(String columnName);

	public abstract void stopReporting();
	
	public abstract void flush();
	
	public abstract Map<String, Integer> makeIdBatch(String tableName,
			String columnName, List<String> values);
	
	public abstract void updateConfig();

	public abstract void reportIpChange(int oldIp, int newIp,
			int localIp, long currentGMTTime);

	public abstract String getIspInfo(String myIp);

	public boolean insertUserReports(int sourceId, String downloadIds, String comment);
	
	public boolean insertIspInfo(int sourceId, String prefix, String ispname);
	
	public boolean insertInterference(int sourceId, String type, long timestamp);

	public boolean insertInterferenceEvent(int sourceId, String type, long timestamp, long downloadId, long droppedTo, float mean);

	public boolean insertTask(int sourceId, long timeStamp, int taskId);

	/**
	 * 
	 */
	public abstract void updateLastFailTime();

	/**
	 * @param l
	 */
	public abstract void setLastFailTime(long l);

	/**
	 * @return
	 */
	public abstract boolean getCanSendData();

	/**
	 * @return
	 */
	public abstract String getDefaultServer();

	/**
	 * @return
	 */
	public abstract String getCurrentUrl();

	/**
	 * @return
	 */
	public abstract Charset getDefaultCharSet();

	/**
	 * @return
	 */
	public abstract boolean isUseGzip();

}