/**
 * Dasu Project
 *
 * File:         ActiveMeasurementEngine.java
 * RCS:          $Id: ActiveMeasurementEngine.java,v 1.4 2010/08/27 05:08:21 mas939 Exp $
 * Description:  ActiveMeasurementEngine class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:28:49 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.drools;

import org.drools.logger.KnowledgeRuntimeLoggerFactory;

import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Util;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The ActiveMeasurementEngine class ...
 */
public class ActiveMeasurementEngine extends RuleBasedEngine {

	public ActiveMeasurementEngine(String filename, Integer taskId) {
		super("ActiveMeasurementEngine", filename);
		this.taskId = taskId;
	}
	
	@Override
	public boolean loadRules() {
		kbase = loadRules(filename);
		ksession = kbase.newStatefulKnowledgeSession();
		
		if (DEBUG)
			logger = KnowledgeRuntimeLoggerFactory.newFileLogger(ksession, "active");
		
		return true;
	}
	
	@Override
	protected void loadFacts() {
		
		
	}
	
	@Override
	protected void destroySession() {
		
		if (ksession != null)
			ksession.dispose();
		
		if (taskId != -1) {
			DasuManager.getPersistentDataManager().getTaskLogHistory().put(taskId, Util.currentGMTTime());
			Statistics.getInstance().commitTask(taskId);
		}

	}

	
}
