/**
 * Dasu Project
 *
 * File:         DasuHttpModule.java
 * RCS:          $Id: DasuScrapeModule.java,v 1.1 2010/11/01 15:37:00 mas939 Exp $
 * Description:  DasuHttpModule class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 15, 2010 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.ono.experiment
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.gudy.azureus2.core3.torrent.TOTorrent;
import org.gudy.azureus2.core3.torrent.TOTorrentException;
import org.gudy.azureus2.core3.torrent.TOTorrentFactory;
import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerFactory;
import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerDataProvider;
import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerException;
import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerListener;
import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerResponse;
import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerResponsePeer;
import org.gudy.azureus2.core3.tracker.client.TRTrackerScraper;
import org.gudy.azureus2.core3.tracker.client.TRTrackerScraperResponse;
import org.gudy.azureus2.core3.util.SystemTime;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadAnnounceResult;
import org.gudy.azureus2.plugins.download.DownloadAnnounceResultPeer;

import com.aelitis.azureus.plugins.dht.DHTPlugin;
import com.aelitis.azureus.plugins.dht.DHTPluginContact;
import com.aelitis.azureus.plugins.dht.DHTPluginOperationListener;
import com.aelitis.azureus.plugins.dht.DHTPluginValue;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.MainGeneric;
import edu.northwestern.dasu.drools.FactScrapeProbeResult;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Pair;



/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The HamecaPingModuleV2 class manages the set of 
 *         Pings performed by the peer.
 */

public class DasuScrapeModule extends DasuProbeModule {	
		
	private class TaskEntry {
		
		public Integer taskId;
		public String hash;
		public String trackerUrl = "";
		public String type = "DHT";
		
		public TaskEntry(String hash, Integer taskId, Map params) {
			this.taskId = taskId;
			this.hash = hash;
						
			if ((params != null) && params.containsKey("type") && 
					validTypes.contains((String) params.get("type")) &&
							params.containsKey("trackerUrl")) {
				
				trackerUrl = (String) params.get("trackerUrl");
				type = (String) params.get("type");
			}

		}
	}
	
	public Hashtable<Long, FactScrapeProbeResult> scrapeResultTable;
	private ArrayList<TaskEntry> toDoScrapeList = new ArrayList<TaskEntry>();
	private static DasuScrapeModule self;
	private HashSet<String> validTypes = 
		new HashSet<String>(Arrays.asList(new String[]{"DHT", "TRACKER_SCRAPE", "TRACKER_ANNOUNCE"}));
	TRTrackerAnnouncer trackerClient;
	
	private static int MIN_INTERVAL = 10; //minimun interval allowed between scrapes to the same server (minutes)
	public Hashtable<String, Long> trackerHistory = new Hashtable<String, Long>();

	
	public DasuScrapeModule() {
		super("DasuScrapeModule");
		this.pi = MainGeneric.getPluginInterface();
		scrapeResultTable = new Hashtable<Long, FactScrapeProbeResult>();

		active = true;
	}
	
	public synchronized static DasuScrapeModule getInstance() {
		if (self == null) {
			self = new DasuScrapeModule();
			self.setDaemon(true);
			return self;
		} else {
			return self;
		}
	}

	@Override
	public void run() {
		
	  while (!MainGeneric.isShuttingDown() && isActive()) {
		 
			TaskEntry taskEntry = null;
			synchronized (toDoScrapeList) {
				while ((toDoScrapeList.size()==0) && !MainGeneric.isShuttingDown()) {
					try {
						toDoScrapeList.wait(sleepInterval*1000);
					} catch (InterruptedException e) {
						self = null;
						return;
					}
				}
				
				if (toDoScrapeList.size() > 0)
					taskEntry = toDoScrapeList.remove(0);
				
				if (DEBUG)
					System.out.println("DasuScrapeModule: Hash " + taskEntry.hash );
			}
			
			if (taskEntry.hash.equals("HALT")) 
				return;
			
			//remove probe history of any tracker over 10 minutes old
			synchronized(trackerHistory) {
				
				Iterator<Entry<String, Long>> iter = trackerHistory.entrySet().iterator();
				while (iter.hasNext()) {
					Entry<String, Long> entry = iter.next();
					if ((System.currentTimeMillis() - entry.getValue())/1000/60 > MIN_INTERVAL)
						iter.remove();
				}
								
			}
			
			probeIp(taskEntry.hash, taskEntry.taskId, taskEntry.trackerUrl, taskEntry.type);


	  } // end while active
		
	  self = null;
	  
	}
		
	@Override
	public void probeAll(List<String> list, Integer taskId) {

		synchronized (toDoScrapeList) {
			for (String hash : list)
				toDoScrapeList.add(new TaskEntry(hash, taskId, null));
			toDoScrapeList.notifyAll();
		}
	}
	
	/*default probe is to the DHT*/
	
	@Override
	public void probeWithPriority(String hash, Integer taskId) {
		
		synchronized (toDoScrapeList) {
			toDoScrapeList.add(new TaskEntry(hash, taskId, null));
			toDoScrapeList.notifyAll();
		}		
	}

	public void probe(String hash, Integer taskId, Map params) {
		
		System.out.println("probe() " + hash + " " + taskId + " " + params);
		
		synchronized (toDoScrapeList) {
			toDoScrapeList.add(new TaskEntry(hash, taskId, params));
			toDoScrapeList.notifyAll();
		}

	}
	
	/*default probe is to the DHT*/
	
	@Override
	public void probe(String hash, Integer taskId) {

		synchronized (toDoScrapeList) {
			toDoScrapeList.add(new TaskEntry(hash, taskId, null));
			toDoScrapeList.notifyAll();
		}
	}
	
	protected Object probeIp(String hash, Integer taskId, String trackerUrl, String type) {
		
		trackerUrl=trackerUrl!=null ? trackerUrl : "";
		
		if (trackerHistory.containsKey(trackerUrl) && (System.currentTimeMillis() - trackerHistory.get(trackerUrl))/1000/60 < MIN_INTERVAL) {
			
				if (DEBUG)
					System.out.println("DasuScrapeModule ERROR: new probe to same trackers must be " + MIN_INTERVAL + " minutes appart");
				
				updateScrapeResultQueue(new FactScrapeProbeResult(hash, type, taskId, trackerUrl));
			
		}
				
		else if (hash.length() == 40) {
			
			if (type.equals("DHT")) {
				if ( ((trackerUrl != null) && (trackerUrl.length() != 0)) || (trackerUrl == null) )
					dhtScrape(hash, taskId, trackerUrl);
				else
					dhtScrape(hash, taskId);
			}
			else if (type.equals("TRACKER_SCRAPE"))
				trackerScrape(hash, taskId, trackerUrl);
			else if (type.equals("TRACKER_ANNOUNCE"))
				trackerAnnounce(hash, taskId, trackerUrl);
			
		}
		else {
			System.err.println("DasuScrapeModule: ERROR: invalid hash");
		}

		return null;
	}
	
	/*default probe is to the DHT*/
	
	@Override
	protected Object probeIp(String hash, Integer taskId) {
		
		return probeIp(hash, taskId, null, "DHT");
		
	}
	
	private void updateScrapeResultQueue(FactScrapeProbeResult result) {
		
		Long id = System.nanoTime();
		scrapeResultTable.put(id, result);
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(
				new Pair<Long, EventType>(id, EventType.SCRAPE_RESULT));
			AMCoordinatorV2.events.notifyAll();
		}
		
		if (result.type.equals("TRACKER_ANNOUNCE"))
			trackerClient.destroy();
		
		if (!result.isError()) {

			synchronized(trackerHistory) {
				trackerHistory.put(result.tracker, System.currentTimeMillis());
			}
			
		}
	}
	
	@Override
	public void stopModule() {
		
		setActive(false);
		synchronized(toDoScrapeList) {
			toDoScrapeList.add(new TaskEntry("HALT", -1, null));
			toDoScrapeList.notify();
		}
	
		self = null;
	}
	
	protected void dhtScrape(final String hexHash, final Integer taskId) {
		dhtScrape(hexHash, taskId, "");
	}
	
	protected void dhtScrape(final String hexHash, final Integer taskId, final String trackerUrl) {
		
		DHTPlugin dht;
		final org.gudy.azureus2.plugins.PluginInterface dht_pi = 
			Main.getPluginInterface().getPluginManager().getPluginInterfaceByClass(DHTPlugin.class);
				
		if ( dht_pi != null ){			
			dht = (DHTPlugin)dht_pi.getPlugin();
					
			System.out.println(	"Tracker announce for 'torrent'" );
			dht.get(hexStringToByteArray(hexHash),
					"Tracker announce for torrent",
					DHTPlugin.FLAG_DOWNLOADING, 30,
					2*60*1000, false, false, 
					new DHTPluginOperationListener(){

						List<String>	addresses 	= new ArrayList<String>();
						List<Integer>	ports		= new ArrayList<Integer>();
						List<Integer>	udp_ports	= new ArrayList<Integer>();
						List<Boolean>	is_seeds	= new ArrayList<Boolean>();
						List<String>	flags		= new ArrayList<String>();
						
						int		seed_count;
						int		leecher_count;

						boolean	complete;
						
						public void complete(byte[] key,
								boolean timeout_occurred) {
							synchronized( this ){
								
								if ( complete ){									
									return;
								}
								
								complete = true;
							}
							
							if (DEBUG) {
								System.out.println("addresses=" + addresses.size() + ", seeds="
									+ seed_count + ", leechers=" + leecher_count);
							
								for (String add : addresses) {
									System.out.println(add);
								}
							}
							
							FactScrapeProbeResult ret = new FactScrapeProbeResult(hexHash,
									"DHT", 60, taskId, addresses, ports, udp_ports, is_seeds, 
									seed_count, leecher_count, trackerUrl);
									
							Statistics.getInstance().addScrapeResult(ret);
							updateScrapeResultQueue(ret);
							
						}

						public void diversified() {
						}

						public void starts(byte[] key) {
						}

						public void valueRead(DHTPluginContact originator,
								DHTPluginValue value) {

							synchronized( this ){
								
								if ( complete ){
									
									return;
								}

								try{										
									String[]	tokens = new String(value.getValue()).split(";");
								
									String	tcp_part = tokens[0].trim();
									
									int	sep = tcp_part.indexOf(':');
									
									String	ip_str		= null;
									String	tcp_port_str;
									
									if ( sep == -1 ){
										
										tcp_port_str = tcp_part;
										
									}else{
										
										ip_str 			= tcp_part.substring( 0, sep );							
										tcp_port_str	= tcp_part.substring( sep+1 );	
									}
									
									int	tcp_port = Integer.parseInt( tcp_port_str );
										
									if ( tcp_port > 0 && tcp_port < 65536 ){
		
										String	flag_str	= null;
										int		udp_port	= -1;
										
										try{
											for (int i=1;i<tokens.length;i++){
											
												String	token = tokens[i].trim();
												
												if ( token.length() > 0 ){
													
													if ( Character.isDigit( token.charAt( 0 ))){
														
														udp_port = Integer.parseInt( token );
														
														if ( udp_port <= 0 || udp_port >=65536 ){
															
															udp_port = -1;
														}
													}else{
														
														flag_str = token;
													}
												}
											}
										}catch( Throwable e ){
										}
									
										addresses.add( 
												ip_str==null?originator.getAddress().getAddress().getHostAddress():ip_str);
										
										ports.add( new Integer( tcp_port ));
										
										udp_ports.add( new Integer( udp_port==-1?originator.getAddress().getPort():udp_port));
										
										flags.add( flag_str );
										
										if (( value.getFlags() & DHTPlugin.FLAG_DOWNLOADING ) == 1 ){
											
											leecher_count++;
											
											is_seeds.add( new Boolean( false ));
		
										}else{
											
											is_seeds.add( new Boolean( true ));
											
											seed_count++;
										}
									}
								
								}catch( Throwable e ){
									
									// in case we get crap back (someone spamming the DHT) just
									// silently ignore
								}
							}
							
						}

						public void valueWritten(DHTPluginContact target,
								DHTPluginValue value) {
						}});
			}

	}

	protected void trackerScrape (final String hexHash, final Integer taskId, final String trackerURL) {

		try {

			TOTorrent torrent = null;
			
			String file = Main.getPluginInterface().getPluginDirectoryName() + java.io.File.separator+ 
				"temp.torrent";
			
			File sourceFile = new File(file);
			FileOutputStream fos = new FileOutputStream(sourceFile);
			DataOutputStream dos = new DataOutputStream(fos);
		    dos.writeInt(2333);
		    dos.writeChars("Hello");	
		    dos.close();
		    fos.close();
		    
			int piece_size = 1024;
			TOTorrent temp = TOTorrentFactory.createFromFileOrDirWithFixedPieceLength(
					sourceFile,
					new URL( "dht://a.b.c/" ),
					piece_size ).create();
			temp.setHashOverride(hexStringToByteArray(hexHash));
			temp.setAnnounceURL(new URL(trackerURL));
			torrent = temp;	
			
			//Scrape
			TRTrackerScraper scraper = Main.getGlobalManager().getTrackerScraper();
			TRTrackerScraperResponse response = scraper.scrape(torrent, true);
			
			if (DEBUG) {
				System.out.println("status: " + response.getStatusString());
				System.out.println("length: " + response.getPeers());
				System.out.println("time next update: " + response.getNextScrapeStartTime());
			}
			
			sourceFile.delete();

			FactScrapeProbeResult ret = null;

			//if the client has not finished but has not returned an erro then wait for 5 seconds to allow the tracker
			//to reponse, else report as error
			if ((response.getStatus() != response.ST_ERROR) && (response.getStatus() != response.ST_ONLINE)) {
				System.out.println("DasuScrapeModule(): Waiting for 5 seconds...");
				Thread.sleep(5*1000);
			}
			
			
			if (response.getStatus() == response.ST_ONLINE) {
			
				int toWait = (int)((response.getNextScrapeStartTime() - SystemTime.getCurrentTime()) / 1000);
				
				ret = new FactScrapeProbeResult(hexHash,
						"TRACKER_SCRAPE", toWait, taskId, 
						new ArrayList<String>(), new ArrayList<Integer>(), new ArrayList<Integer>(), 
						new ArrayList<Boolean>(), response.getSeeds(), response.getPeers(), trackerURL);
			}
			else
				ret = new FactScrapeProbeResult(hexHash, "TRACKER_SCRAPE", taskId, trackerURL);
			
					
			Statistics.getInstance().addScrapeResult(ret);
			updateScrapeResultQueue(ret);

		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (TOTorrentException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	protected void trackerAnnounce (final String hexHash, final Integer taskId, final String trackerURL) {
		
		try {

			TOTorrent torrent = null;
			
			String file = Main.getPluginInterface().getPluginDirectoryName() + java.io.File.separator + 
				"temp.torrent";
			
			File sourceFile = new File(file);
			FileOutputStream fos = new FileOutputStream(sourceFile);
			DataOutputStream dos = new DataOutputStream(fos);
		    dos.writeInt(2333);
		    dos.writeChars("Hello");	
		    dos.close();
		    fos.close();
		    
			int piece_size = 1024;
			TOTorrent temp = TOTorrentFactory.createFromFileOrDirWithFixedPieceLength(
					sourceFile,
					new URL( "dht://a.b.c/" ),
					piece_size ).create();
			temp.setHashOverride(hexStringToByteArray(hexHash));
			temp.setAnnounceURL(new URL(trackerURL));
			torrent = temp;	
			
			//Announce
			
			trackerClient = TRTrackerAnnouncerFactory.create( torrent, null);
			trackerClient.addListener(new TRTrackerAnnouncerListener() {
			
		
				public void receivedTrackerResponse(
						TRTrackerAnnouncerResponse response) {
					
					FactScrapeProbeResult ret = null;
					int status = response.getStatus();
					
					if (DEBUG) {
						System.out.println("receivedTrackerResponse()");
						System.out.println("status: " + response.getStatusString());
						if (( (status != TRTrackerAnnouncerResponse.ST_OFFLINE) 
								&& (status != TRTrackerAnnouncerResponse.ST_REPORTED_ERROR)) ) {
							System.out.println("length: " + response.getPeers().length);
						}
						System.out.println("time next update: " + response.getTimeToWait());
						response.print();
					}

					List<String>	addresses 	= new ArrayList<String>();
					List<Integer>	ports		= new ArrayList<Integer>();
					List<Integer>	udp_ports	= new ArrayList<Integer>();
					List<Boolean>	is_seeds	= new ArrayList<Boolean>();

					if (( (status != TRTrackerAnnouncerResponse.ST_OFFLINE) 
							&& (status != TRTrackerAnnouncerResponse.ST_REPORTED_ERROR)) ) {
						
						for (TRTrackerAnnouncerResponsePeer p : response.getPeers()) {
							addresses.add(p.getAddress());
							ports.add(p.getPort());
							udp_ports.add(p.getUDPPort());
						}
						
						ret = new FactScrapeProbeResult(hexHash,
								"TRACKER_ANNOUNCE", response.getTimeToWait(), taskId, 
								addresses, ports, udp_ports, is_seeds, 
								response.getScrapeCompleteCount(), response.getScrapeIncompleteCount(), 
								trackerURL);
					}
					
					else
						ret = new FactScrapeProbeResult(hexHash, "TRACKER_ANNOUNCE", taskId, trackerURL);
												
					Statistics.getInstance().addScrapeResult(ret);
					updateScrapeResultQueue(ret);
		
				}
		
				public void urlChanged(TRTrackerAnnouncer announcer,
						URL old_url, URL new_url, boolean explicit) {
				}
		
				public void urlRefresh() {
				}});
			
			trackerClient.setAnnounceDataProvider(
					new TRTrackerAnnouncerDataProvider()
					{
						public String
						getName()
						{
							return( "trackerClient" );
						}
						
						public long
						getTotalSent()
						{
							return( 0 );
						}
						
						public long
						getTotalReceived()
						{
							return( 0 );
						}
						
		    			public long 
		    			getFailedHashCheck() 
		    			{
		    				return( 0 );
		    			}
		    			
						public long
						getRemaining()
						{
							return( 0 );
						}
						
						public String
						getExtensions()
						{
							return( null );
						}
						
						public int
						getMaxNewConnectionsAllowed()
						{							
							return (50);	
						}
						
						public int 
						getUploadSpeedKBSec(
							boolean estimate) 
						{
							return 0;
						}
						
						public int 
						getCryptoLevel() 
						{
							return 0;
						}
						
						public boolean
						isPeerSourceEnabled(
							String		peer_source )
						{
							return( true );
						}
						
						public void
						setPeerSources(
							String[]	sources )
						{
						}

						public int getConnectedConnectionCount() {
							// TODO Auto-generated method stub
							return 0;
						}

						public int getPendingConnectionCount() {
							// TODO Auto-generated method stub
							return 0;
						}
					});
			
			if ( trackerClient != null) {
				trackerClient.setTrackerURL(new URL(trackerURL));
				trackerClient.update( true );
			}
			else {
				//There was a problem creating the trackerClient 
				//insert empty resultset 
				FactScrapeProbeResult ret = new FactScrapeProbeResult(hexHash, "TRACKER_ANNOUNCE", taskId, trackerURL);

				Statistics.getInstance().addScrapeResult(ret);
				updateScrapeResultQueue(ret);
				
			}
			
			//delete temp file before you are done
			sourceFile.delete();
		
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (TOTorrentException e) {
			e.printStackTrace();
		} catch (TRTrackerAnnouncerException e) {
			e.printStackTrace();
		}
			
	}
	
	public  String getHexString(byte[] b) {
		
		String result = "";
		for (int i=0; i < b.length; i++) {
			result +=
		          Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 );
		}
		return result;
	}
	
	private static byte[] hexStringToByteArray(String s) {
		
		byte[] b = new byte[s.length() / 2];
	    for (int i = 0; i < b.length; i++){
	    	int index = i * 2;
	    	int v = Integer.parseInt(s.substring(index, index + 2), 16);
	    	b[i] = (byte)v;
	    }
	    return b;
	}

	
}
