/**
 * Dasu Project
 *
 * File:         MovingWindow.java
 * RCS:          $Id: MovingWindow.java,v 1.2 2010/11/01 15:37:00 mas939 Exp $
 * Description:  MovingWindow class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 21, 2010 at 2:20:33 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import java.util.ArrayList;
import java.util.Collection;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The MovingWindow class ...
 */

public class MovingWindow {
	
	ArrayList<Float> mean;
	ArrayList<Long> time;
	ArrayList<Float> stdDev;
	ArrayList<Float> signal;
	ArrayList<Float> varianceFromStdev; 
	
	int windowSize;
	
	public MovingWindow(Collection<Float> signal, Collection<Long> time, int wz) {
		
		this.windowSize = wz;
		this.mean = (ArrayList<Float>) SignalManipulation.getAverageCollection(signal, wz);
		this.stdDev = (ArrayList<Float>) SignalManipulation.getStdDevCollection(signal, wz);
		this.varianceFromStdev = (ArrayList<Float>) SignalManipulation.varianceFromStdev(signal, mean, stdDev, wz);
		
		this.signal = new ArrayList<Float>();
		this.time = new ArrayList<Long>();
		for (int i=wz; i<mean.size(); i++) {
			this.signal.add( ((ArrayList<Float>) signal).get(i) );
			this.time.add( ((ArrayList<Long>) time).get(i) );
		}
		
	}
	
	public MovingWindow(Float[] signal, Long[] time, int wz) {
		
		this.windowSize = wz;
		this.mean = (ArrayList<Float>) SignalManipulation.getAverageCollection(signal, wz);
		this.stdDev = (ArrayList<Float>) SignalManipulation.getStdDevCollection(signal, wz);
		this.varianceFromStdev = (ArrayList<Float>) SignalManipulation.varianceFromStdev(signal, mean, stdDev, wz);
		
		mean.remove(mean.size()-1);
		stdDev.remove(stdDev.size()-1);
		varianceFromStdev.remove(varianceFromStdev.size()-1);
		
		this.signal = new ArrayList<Float>();
		this.time = new ArrayList<Long>();
		for (int i=0; i<mean.size(); i++) {
			this.signal.add( signal[i+wz] );
			this.time.add( time[i+wz] );
		}
				
	}
	
	@Override
	public String toString() {
		
		return "windowSize:  " + windowSize + "\ntime: " + time + "\nsignal: " + signal + "\nmean: " + mean +
			"\nstdev: " + stdDev + "\nvarianceFromDev: " + varianceFromStdev;  
	}

	public ArrayList<Float> getMean() {
		return mean;
	}

	public ArrayList<Long> getTime() {
		return time;
	}

	public ArrayList<Float> getStdDev() {
		return stdDev;
	}

	public ArrayList<Float> getSignal() {
		return signal;
	}

	public ArrayList<Float> getVarianceFromStdev() {
		return varianceFromStdev;
	}

	public int getWindowSize() {
		return windowSize;
	}

	public void setMean(ArrayList<Float> mean) {
		this.mean = mean;
	}

	public void setTime(ArrayList<Long> time) {
		this.time = time;
	}

	public void setStdDev(ArrayList<Float> stdDev) {
		this.stdDev = stdDev;
	}

	public void setSignal(ArrayList<Float> signal) {
		this.signal = signal;
	}

	public void setVarianceFromStdev(ArrayList<Float> varianceFromStdev) {
		this.varianceFromStdev = varianceFromStdev;
	}

	public void setWindowSize(int windowSize) {
		this.windowSize = windowSize;
	}
	
}