/**
 * Dasu Project
 *
 * File:         NetstatBanwidthPerformer.java
 * RCS:          $Id: NetstatBanwidthPerformer.java,v 1.2 2010/11/01 15:37:00 mas939 Exp $
 * Description:  NetstatBanwidthPerformer class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Sep 30, 2010 at 11:57:52 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.PluginInterface;
import com.azureus.plugins.aznetmon.main.RuntimeExec;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.measurement.NetstatBWStats.NetstatBWData;

import java.util.*;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The NetstatBanwidthPerformer class ...
 */


public class NetstatBanwidthPerformer implements UTTimerEventPerformer {

    PluginInterface pi=null;
    public String platform;
    private String eth;
    UTTimerEvent timerEvent = null;
    private boolean DEBUG = false;

    //eth is the interface name I want to use
    public NetstatBanwidthPerformer(PluginInterface pluginInterface, String eth){

        this.pi = pluginInterface;
        this.eth = eth;
        this.platform = Main.getPlatform();
        		
    }

    public void cancelTimer() {
    			
    	if (DEBUG)
    		System.out.println("MARIO: Canceling timeEvent!");
    	
    	if (timerEvent != null) {    		
    		timerEvent.cancel();
    	}
    }
    
    public void perform(UTTimerEvent utTimerEvent)
    {
    	
   		timerEvent = utTimerEvent;
   		
   		if (DEBUG)
   			System.out.println("MARIO: NETSTATBANDWIDTHPERF PERFORM!: " + utTimerEvent.toString());
   		
        try{
            
            String[] cmd = new String[3];
            
            if (platform.equals("windows")) {
	            cmd[0] = "cmd.exe";
	            cmd[1] = "/C";
	            cmd[2] = "netstat -e";
            }
            else if (platform.equals("OSX")) {

            	cmd[0] = "/usr/sbin/netstat";
            	cmd[1] = "-ib";
            	cmd[2] = "\0";
            }
            else if (platform.equals("linux")) {
            	
            	cmd = new String[2];
            	cmd[0] = "/sbin/ifconfig";
            	cmd[1] = eth;
            }

            RuntimeExec re = new RuntimeExec(cmd);

            re.exec();

            try{Thread.sleep(1000);}catch(Exception e){}

	            String output = re.getStdOut();
	            String err = re.getStdErr();
	            
	         if (DEBUG) {
	       	 	System.out.println("NETSTAT: interface: " + eth);
	       	 	System.out.println("NETSTAT: platform: " + platform);
	       	 	System.out.println("NETSTAT: cmd: " + Arrays.asList(cmd));
	       	 	System.out.println("NETSTAT: err: " + err);
	            System.out.println("NETSTAT: output: " + output);
            }

            if( output != null ){

            	parseOutput(output);

            }//if

        }catch(Exception e){

        	NetstatBWStats stats = NetstatBWStats.getInstance();
            stats.setStatus( e.toString() );
        }


    }//perform

    
    public NetstatBWData getData(String platform)
    {
    	
        try{
            
            String[] cmd = new String[3];
            
            if (platform.equals("windows")) {
	            cmd[0] = "cmd.exe";
	            cmd[1] = "/C";
	            cmd[2] = "netstat -e";
            }
            else if (platform.equals("OSX")) {
            	cmd[0] = "/usr/sbin/netstat";
            	cmd[1] = "-ib";
            	cmd[2] = "\0";
            }
            else if (platform.equals("linux")) {
            	cmd = new String[2];
            	cmd[0] = "/sbin/ifconfig";
            	cmd[1] = eth;
            }
            

            RuntimeExec re = new RuntimeExec(cmd);

            re.exec();

            try{Thread.sleep(1000);}catch(Exception e){}

            String output = re.getStdOut();
            //System.out.println("Output: " + output);

            if( output != null ){

            	return getParsedOutput(output);

            }//if

        }catch(Exception e){

            System.out.println(e.toString());
        }
        
        return null;

    }//perform
    
    /**
     *
     * @param output - expects results of netstat -s for a windows machine.
     */
    void parseOutput(String output) {
        String[] lines = output.split("\n");
        if( lines.length > 0 ){

        	NetstatBWStats.NetstatBWData bwData;
        	if (platform.equals("OSX"))
        		bwData = parseOSX(lines);
        	else if (platform.equals("windows"))
        		bwData = parse(lines);
        	else
        		bwData = parseLinux(lines);


        	/****** DO I REALLY NEED THIS? 
        	 
        	if ( isResultEmpty( bwData )) {
              //Likely the default language of computer isn't English, try backup plan.
            	if (platform.equals("windows"))
            		pktData = parseByOrder(lines);
            	else if (platform.equals("OSX"))
            		pktData = parseByOrderOSX(lines);        		
        	}
        	
        	*****/
        	
            //Get the deltas.
            NetstatBWStats stats = NetstatBWStats.getInstance();
            NetstatBWStats.NetstatBWData recent = stats.getMostRecent();

            if( recent != null ){

                bwData.calculate( recent );

            }

        	stats.add( bwData );

        }//if
    }

    NetstatBWData getParsedOutput(String output) {

    	String[] lines = output.split("\n");
        if( lines.length > 0 ){

        	NetstatBWStats.NetstatBWData bwData;
        	if (platform.equals("OSX"))
        		bwData = parseOSX(lines);
        	else if (platform.equals("linux"))
        		bwData = parseLinux(lines);
        	else 
        		bwData = parse(lines);

        	/****** DO I REALLY NEED THIS? 
        	 
        	if ( isResultEmpty( bwData )) {
              //Likely the default language of computer isn't English, try backup plan.
            	if (platform.equals("windows"))
            		pktData = parseByOrder(lines);
            	else if (platform.equals("OSX"))
            		pktData = parseByOrderOSX(lines);        		
        	}
        	
        	*****/
        	
            //Get the deltas.
            NetstatBWStats stats = NetstatBWStats.getInstance();
            NetstatBWStats.NetstatBWData recent = stats.getMostRecent();

            if( recent != null ){

                bwData.calculate( recent );

            }

        	return bwData ;

        }//if
        
        return null;
    }
    
    
    NetstatBWStats.NetstatBWData parse(String[] lines) {

    	 NetstatBWStats.NetstatBWData bwData = new NetstatBWStats.NetstatBWData();
         
         for(int i=0;i<lines.length;i++){
         	
        	 
         	if (!lines[i].contains("Bytes"))
        		continue;        		
        	
        	lines[i] = lines[i].replaceAll("\\s+", " ");

        	String[] value = lines[i].split(" ");
            
            if( value.length >= 2 ){
                
            	System.out.println(lines[i]);
            	bwData.bytesReceived = Long.parseLong( value[1] );
            	bwData.bytesSent = Long.parseLong( value[2]);
            	System.out.println("bytesReceived: " + bwData.bytesReceived + " bytesSent: " + bwData.bytesSent);
                
            }//if

        }//for

        return bwData;
    }

    NetstatBWStats.NetstatBWData parseOSX(String[] lines) {

   	 	NetstatBWStats.NetstatBWData bwData = new NetstatBWStats.NetstatBWData(); 
    	
        for(int i=0;i<lines.length;i++){
        	
        	if (!lines[i].contains("Link") || !lines[i].contains("<"))
        		continue;
        		
        	if (!lines[i].contains(eth))
        		continue;
        	        	
        	lines[i] = lines[i].replaceAll("\\s+", " ");

        	String[] value = lines[i].split(" ");
            
            if( value.length >= 2 ){
            
            	bwData.bytesReceived = Long.parseLong( value[6]);
            	bwData.bytesSent = Long.parseLong( value[9] );
                
            }//if

        }//for

        return bwData;
   }
    
    NetstatBWStats.NetstatBWData parseLinux(String[] lines) {

   	 	NetstatBWStats.NetstatBWData bwData = new NetstatBWStats.NetstatBWData(); 
    	
        for(int i=0;i<lines.length;i++){
        	
        	System.out.println("NETSTAT: " + lines[i]);
        	
        	if (!lines[i].contains("RX bytes"))
        		continue;
        	        	
        	lines[i] = lines[i].replaceAll("\\s+", " ");

        	String[] value = lines[i].split(" ");
            
            if( value.length >= 2 ){
            
            	bwData.bytesReceived = Long.parseLong(value[2].split(":")[1]);
            	bwData.bytesSent = Long.parseLong(value[6].split(":")[1]);
            	//bwData.bytesReceived = Long.parseLong( value[1] ) * Long.parseLong( value[3] );
            	//bwData.bytesSent = Long.parseLong( value[1] ) * Long.parseLong( value[7] );
                
            }//if

        }//for

        return bwData;
   }

    /**
     * toIntArray.
     * @param l - Really a List<Integer> in 1.5 symantics.
     * @return - int[]
     */
    int[] toIntArray(List l){

        if(l==null){
            return new int[0];
        }

        int size = l.size();
        int[] retVal = new int[size];

        for(int i=0;i<size;i++){
            retVal[i] = ((Integer) l.get(i)).intValue();
        }

        return retVal;        
    }//toIntArray

    /**
    RSTPacketStats.RSTPacketData makeGoodCandidate( List l ){
        RSTPacketStats.RSTPacketData pktData = new RSTPacketStats.RSTPacketData();

        pktData.nActiveOpens       = ((Integer) l.get(0)).intValue();
        pktData.nPassiveOpens      = ((Integer) l.get(1)).intValue();
        pktData.nFailedConnAttempt = ((Integer) l.get(2)).intValue();
        pktData.nConnReset         = ((Integer) l.get(3)).intValue();
        pktData.nCurrentOpen       = ((Integer) l.get(4)).intValue();

        pktData.msg="candidate";

        return pktData;
    }
	**/

    /**
     * Might want to try an alternate way of collection the data.
     * @param bwData -
     * @return - true if all columns are zero.
     */
    boolean isResultEmpty(NetstatBWStats.NetstatBWData bwData ){

        if( bwData.bytesReceived==-1 &&
        		bwData.bytesSent==-1 ) {
            return true;
        }

        return false;
    }


    
    ///*** testing ***/

    /**
     * For testing.
     * @param args
     */
    public static void main(String[] args)
    {

    	String eth = "en1";
    	NetstatBanwidthPerformer bwperf = new NetstatBanwidthPerformer(null, eth);

        UTTimerEvent empty = new UTTimerEvent(){

            public void cancel() {

            }
        };

        bwperf.perform(empty);
        
        try{Thread.sleep(5 * 1000);}catch(Exception e){}

        bwperf.perform(empty);

        System.out.println(NetstatBWStats.getInstance().getMostRecent().bytesReceived);

        
    }//main


}//class

