package edu.northwestern.dasu.measurement;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import edu.northwestern.dasu.MainGeneric;

/*return a TreeMap with the schedule for the AM to follow*/

public class ProbeScheduler {

	public class ProbeEntry {
	
		Integer priority;
		String ip;
		String probeType;
		
		ProbeEntry(Integer prior, String ip, String type) {
			this.priority = prior;
			this.ip = ip;
			this.probeType = type;
		}
		
		public void print() {
			System.out.println( this.probeType + " " + this.ip + " " + this.priority);
		}
	}
	
	/*Map<TimeStamp begin of 20 mins interval, List<Tasks>>*/
	Map<Long, List<ProbeEntry>> schedule;
	
	ProbeScheduler() {		
		schedule = new TreeMap<Long, List<ProbeEntry>>();
	}
	
	public int countProbes(List<ProbeEntry> list, String probeType) {
	
		int count = 0;
		for ( ProbeEntry l : list) {
			if (l.probeType.equals(probeType))
				count++;
		}
		return count;		
	}
	
	@SuppressWarnings("deprecation")
	public void createSched (Set<ProbeTask> pingList,
			Set<ProbeTask> trList, Set<ProbeTask> pathloadList ) {

		Map<Long, ProbeEntry> retMap = new TreeMap<Long, ProbeEntry>();
		addTasks(pingList, "ping");
		addTasks(trList, "tr");
		addTasks(pathloadList, "pathload");
		
	}
	
	/* add tasks to the schedule in intervals of 20 minutes, if the 
	 * list of tasks is not short enough to fit in the time interval
	 * while complying with the restrictions, it schedules the remaining
	 * probes to be executed past the interval's endTime 
	 */
	@SuppressWarnings("deprecation")
	public void addTasks(Set<ProbeTask> probeList, String probeType) {
		
		for ( ProbeTask pt : probeList) {
			Date d = new Date(pt.startTime/1000 * 1000);
			d.setMinutes( (d.getMinutes() / 20) * 20);
			long interval = d.getTime();
			
			int prior = pt.priority;
			for ( String ip : pt.ipsToProbe ) {
				
				ProbeEntry t = new ProbeEntry(prior, ip, probeType);

				if (!schedule.containsKey(interval))
					schedule.put(interval, new ArrayList<ProbeEntry>());


				do {

					//System.out.println(interval + " " + probeType + " " + MainGeneric.getMaxNumProbes(probeType));
					if ( countProbes( schedule.get(interval), probeType) < MainGeneric.getMaxNumProbes(probeType)) {
						schedule.get(interval).add(t);
					}
					
					else {
						interval += MainGeneric.getProbeInterval() * 1000*60;
						if (!schedule.containsKey(interval))
							schedule.put(interval, new ArrayList<ProbeEntry>());
					}
					
				} while ( !schedule.get(interval).contains(t));

				
			}
			
		}
	}
	
	public List<ProbeEntry> getCurrentSchedule(Long time) {
		if (schedule.containsKey(time))
			return schedule.get(time);
		
		return new ArrayList<ProbeEntry>();
		
	}
	
	public void printSchedule() {
		for ( long key : schedule.keySet()) {
			System.out.println(key + ": ");

			for ( ProbeEntry task : schedule.get(key)) {
				task.print();
			}
			System.out.println();
		}
	}
	
}
