/**
 * Dasu Project
 *
 * File:         DownloadStats.java
 * RCS:          $Id: DynamicLocalPeerStat.java,v 1.3 2010/10/22 19:51:35 mas939 Exp $
 * Description:  DownloadStats class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 10, 2007 at 1:39:15 PM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadManager;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.MainGeneric;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 *
 * localPeerStatDynamic, every 30 seconds, peerId, time, uploadRate (at that time), 
 * downloadRate (at that time), maxDonwloadRate, maxUploadRate, autoSeed
 */

public class DynamicLocalPeerStat  implements Cloneable{
	
	public long currentTime = Util.currentGMTTime();
	public int uploadRate;  	/*instantaneous upload rate*/
	public int downloadRate;	/*instantaneous download rate*/
	public int maxUploadRate;
	public int maxDownloadRate;
	public boolean autoSpeed;
	public int rst;
	public int numConnections;
	public int passiveOpens;
	public int activeOpens;
	public int failedConns;
	public int numTorrents = 0;
	public int numLeechingTorrents = 0;
	public int numSeedingTorrents = 0;
	public int numConnectedSeeds = 0;
	public int numConnectedLeechers = 0;
	public long bytesSentLpsd;
	public long bytesReceivedLpsd;
	public long swarmPeerSpeed;
	public long bytesSentNetstat;
	public long bytesReceivedNetstat;
	
	public int maxUploadRateLimitAllActiveTorrents = 0;
	public int maxDownloadRateLimitAllActiveTorrents = 0;

	public DynamicLocalPeerStat() {
		
		DownloadManager dm = MainGeneric.getPluginInterface().getDownloadManager();
				
		downloadRate = dm.getStats().getDataReceiveRate();;
		uploadRate = dm.getStats().getDataSendRate();
		maxDownloadRate = COConfigurationManager.getIntParameter("Max Download Speed KBs")*1000;
		maxUploadRate = COConfigurationManager.getIntParameter("Max Upload Speed KBs")*1000;
		autoSpeed = checkAutoSpeed();
		numTorrents = dm.getDownloads().length;
		bytesSentLpsd = dm.getStats().getDataBytesSent();
		bytesReceivedLpsd = dm.getStats().getDataBytesReceived();
		swarmPeerSpeed = Main.getGlobalManager().getStats().getTotalSwarmsPeerRate(true, false);
		

		numConnectedSeeds = 0;
		numConnectedLeechers = 0;
		for (Download d : dm.getDownloads()) {
			
			maxUploadRateLimitAllActiveTorrents += d.getUploadRateLimitBytesPerSecond();

			if (d.getState() == Download.ST_SEEDING) {
				numSeedingTorrents++;
			}
			if (d.getState() == Download.ST_DOWNLOADING) {
				numLeechingTorrents++;
				maxDownloadRateLimitAllActiveTorrents += d.getDownloadRateLimitBytesPerSecond();
			}
			
			if (d.getPeerManager() != null) {
				numConnectedSeeds += d.getPeerManager().getStats().getConnectedSeeds();
				numConnectedLeechers += d.getPeerManager().getStats().getConnectedLeechers();
			}
		}
		
		/****

		TRANSFER SECTION
		
 		"Max Uploads Seeding" => "Alternate default when seeding"
		"enable.seedingonly.maxuploads" => "Alternate default when seeding"
		  
		"Max Uploads" => "Max upload slots per torrent default"
		
		"Max.Peer.Connections.Per.Torrent" => "Max connections per torrent default [0: unlimited]"

		"Max.Peer.Connections.Per.Torrent.When.Seeding.Enable" => 
		"Max.Peer.Connections.Per.Torrent.When.Seeding" => 
		
		"Max.Peer.Connections.Total" => "Max connections globally [0: unlimited]"

		"Max Seeds Per Torrent" => "Max seeds per torrent default [0: unlimited]"
		
		"Use Lazy Bitfield" => "Use lazy bitfield (helps seeding on networks employing bitfield-based blocking)"
		
		CONNECTION SECTION
		
		"TCP.Listen.Port"
		"UDP.Listen.Port"
		
		"network.transport.encrypted.require"
		"network.transport.encrypted.min_level"
		"network.transport.encrypted.fallback.outgoing"
		"network.transport.encrypted.fallback.incoming"
		"network.transport.encrypted.use.crypto.port"
		
		****/
		

	}

	private boolean checkAutoSpeed() {
		String config = "";
		if ( COConfigurationManager.getBooleanParameter(TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY)){
			
			config = ( TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY );
		} else {
		
	    	if ( MainGeneric.getPluginInterface().getDownloadManager().isSeedingOnly()){
	        	
	    		config = ( TransferSpeedValidator.AUTO_UPLOAD_SEEDING_ENABLED_CONFIGKEY );
	        	
	      	}else{
	      		
	      		config = ( TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY );
	      	}
		}
		
		return COConfigurationManager.getBooleanParameter(config);
	}
	
    public Object clone() {

   	 try {
             return super.clone();
         }
   	 catch( CloneNotSupportedException e ) {
             return null;
         }
     } 

}