/**
 * Dasu Project
 *
 * File:         DasuView.java
 * RCS:          $Id: DasuView.java,v 1.29 2010/11/01 15:37:00 mas939 Exp $
 * Description:  DasuView class (see below)
 * Author:       Zachary Bischof
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 26, 2010 at 12:21:13 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.ui
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.ui;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.internat.MessageText;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.ui.swt.Messages;
import org.gudy.azureus2.ui.swt.Utils;
import org.gudy.azureus2.ui.swt.components.LinkLabel;
import org.gudy.azureus2.ui.swt.components.graphics.PingGraphic;
import org.gudy.azureus2.ui.swt.components.graphics.ScaledGraphic;
import org.gudy.azureus2.ui.swt.components.graphics.SpeedGraphic;
import org.gudy.azureus2.ui.swt.config.BooleanParameter;
import org.gudy.azureus2.ui.swt.mainwindow.Colors;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
import org.gudy.azureus2.ui.swt.views.AbstractIView;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.PersistentData;
import edu.northwestern.dasu.asn.NetworkAdminASNLookupImpl;
import edu.northwestern.dasu.drools.FactFireAction;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.SourceType;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Pair;
import edu.northwestern.dasu.util.Util;

/**
 * @author Zachary Bischof &lt;Zachary.Bischof@eecs.northwestern.edu&gt;
 *
 * The DasuView class ...
 */

public class DasuView extends AbstractIView implements UISWTViewEventListener {

	public static final String VIEWID = "Dasu";
	private static DasuView self;
	
	private ScrolledComposite scrollPanel;
	private Composite panel;
	private Shell shell;
	private Display display;
	private Composite descriptionPanel;
	private Composite activeTestsPanel;
	private Group interferenceLog;
	private BooleanParameter enableDasu;
	
	private boolean isCreated = false;
	private ArrayList<HVLogEntry> interferenceNotices = null;
	private Table interferenceTable = null;
	private PluginInterface pi;
	Long dotTorrentTestTimer = -1L;
	Button dotTorrentTestButton = null;
	Button upnpTestButton = null;
    Button enterDescriptionButton = null;
	Long upnpTestTimer = -1L;
	private Button configIspButton;
	private String interferenceReportTorrentIds;
	private String interferenceReportComments;
	private String ispName = null;
	private PersistentData persistentDataManager = null;
	private Shell commentShell;
	private Text commentText;
	private List torrentComboList;
	private Shell ispShell;
	private Combo ispCombo;
	private Text ispText;
	private Label ispLabel;
	private String reportedISP = null;
	
	
	public DasuView( PluginInterface pi ) {
		this.pi = pi;
		init();
		self = this;
	}
	
	public synchronized static DasuView getInstance() {
			return self;
	}

	public void disable() {
	
		upnpTestButton.setEnabled(false);
		dotTorrentTestButton.setEnabled(false);
		interferenceTable.setEnabled(false);
		enterDescriptionButton.setEnabled(false);
	}
	
	public void enable() {
	
		if (configIspButton.isEnabled()) {
			if ((System.currentTimeMillis() - dotTorrentTestTimer > 60*60*1000))
					dotTorrentTestButton.setEnabled(true);
				
				if ((System.currentTimeMillis() - upnpTestTimer > 60*60*1000))
						upnpTestButton.setEnabled(true);
		
				interferenceTable.setEnabled(true);
				enterDescriptionButton.setEnabled(true);
		}
	}

	private void init() {
		interferenceNotices = new ArrayList<HVLogEntry>();
	}
	
	public void refreshIspInfo() {
		
		if ((persistentDataManager == null) && (DasuManager.isInitialized)) {
			persistentDataManager = DasuManager.getPersistentDataManager();
		
			reportedISP = persistentDataManager.getIspInfo().getKey();			
			ispName = reportedISP;
			if(ispName == null){
				ispName = "unknown";
			}
			ispLabel.setEnabled(true);
			ispLabel.setText("Current ISP: " + ispName);
			configIspButton.setEnabled(true);
			enable();
			
		}
	}

	public void initialize(Composite composite) {
		
		// <panel background>
		scrollPanel = new ScrolledComposite(composite,SWT.V_SCROLL|SWT.H_SCROLL);
		scrollPanel.setExpandHorizontal(true);
		scrollPanel.setExpandVertical(true);
		scrollPanel.setMinSize(800, 600);
		shell = composite.getShell();
		panel = new Composite(scrollPanel, SWT.NONE);
		panel.setLayout(new GridLayout());
		display = composite.getDisplay();
		// </panel background>
		
		GridData gridData;
		//gridData.horizontalSpan =2;
		
		Composite textRegion = new Composite(panel, SWT.NONE);
		gridData = new GridData();
		gridData.widthHint = 800;
		
		textRegion.setLayoutData(gridData);
		
		textRegion.setLayout(new GridLayout());
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.widthHint = 800;
		Label desc = new Label(textRegion, SWT.WRAP);
		String text = pi.getUtilities().getLocaleUtilities().getLocalisedMessageText("Dasu.description");
		desc.setText(text);
		
		desc.setLayoutData(gridData);
		
		// <DasuView Aqualab Labels>
		gridData = new GridData();
		GridLayout layout = new GridLayout();
		Composite header = new Composite(panel, SWT.NONE);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		header.setLayoutData(gridData);
		layout.numColumns = 2;
		
		header.setLayout(layout);
		
		layout = new GridLayout();
		
		
		new LinkLabel(header, new GridData(), "Aqualab.label.forums", "http://forums.aqua-lab.org");
		
        // <Comment button>
        
        gridData = new GridData();
        gridData.verticalAlignment = GridData.FILL;
        gridData.verticalSpan = 2;
        gridData.horizontalAlignment = SWT.RIGHT;
        gridData.grabExcessHorizontalSpace = true;
		enterDescriptionButton = new Button(header, 0);
		enterDescriptionButton.setBackground(Colors.fadedYellow);
		
        enterDescriptionButton.setLayoutData( gridData );
        enterDescriptionButton.setText( "Report Undetected Interference" );
        enterDescriptionButton.setToolTipText( "If you suspect you are experiencing a type of interference that Dasu does not detect, please let us know!" );
        enterDescriptionButton.addSelectionListener( addCommentAdapter );
        enterDescriptionButton.setEnabled(false);
               
		// <Comment button>
        new LinkLabel(header, new GridData(), "Aqualab.label.aqualab", "http://aqualab.cs.northwestern.edu");
		// </DasuView Aqualab Labels>
			
		
		// <ActiveTest Buttons>
		activeTestsPanel = new Composite(panel, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridData = new GridData(GridData.FILL_HORIZONTAL);		
		gridLayout.numColumns=3;
		activeTestsPanel.setLayout(gridLayout);
		activeTestsPanel.setLayoutData(gridData);
		
		dotTorrentTestButton = new Button(activeTestsPanel, 0);
        gridData = new GridData();
        dotTorrentTestButton.setLayoutData( gridData );
        dotTorrentTestButton.setText( "Launch .torrent download test" );
        dotTorrentTestButton.setToolTipText( ".torrent download test will be launched as soon as possible" );
        dotTorrentTestButton.addSelectionListener( dotTorrentTestAdapter );
        dotTorrentTestButton.setEnabled(false);
        
        upnpTestButton = new Button(activeTestsPanel, 0);
        gridData = new GridData();
        upnpTestButton.setLayoutData( gridData );
        upnpTestButton.setText( "Launch port blocking test" );
        upnpTestButton.setToolTipText( "port blocking test will be launched as soon as possible" );
        upnpTestButton.addSelectionListener( upnpTestAdapter );
        upnpTestButton.setEnabled(false);
        // </ActiveTest Buttons>      		
        		        
        Composite configIspRegion = new Composite(panel, SWT.NONE);
        gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        configIspRegion.setLayout(gridLayout);
        configIspRegion.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		configIspButton = new Button(configIspRegion, SWT.NONE);
		configIspButton.setText("Change ISP");
		configIspButton.setToolTipText( "Select or enter the name of your ISP");
		configIspButton.addSelectionListener(changeIspAdapter);
		
		ispLabel = new Label(configIspRegion, SWT.WRAP);
		ispName = reportedISP;
		if(ispName == null) {
			ispName = "loading...";
			configIspButton.setEnabled(false);
			ispLabel.setEnabled(false);
		}
		ispLabel.setText("Current ISP: " + ispName);
        gridData = new GridData();
        gridData.widthHint =700;
        ispLabel.setLayoutData(gridData);
		// <interferenceTable>
		interferenceLog = new Group(panel, SWT.NULL);
		Messages.setLanguageText(interferenceLog, "Dasu.view.log.header");
		gridData = new GridData();
		gridData.widthHint=600;
		String url = DasuManager.getInstance().getNewRandomDasuUrl();
		new LinkLabel(interferenceLog, gridData, "Aqualab.label.dasu.interferencetypes.label", 
				url + DasuConfiguration.INTERF_TYPES_SCRIPT + "?version="+DasuConfiguration.getInstance().getInferenceVersion());
		gridData = new GridData(GridData.FILL_HORIZONTAL);		
		gridData.heightHint=250;
		interferenceLog.setLayoutData(gridData);
		interferenceLog.setLayout(layout);		
		//Messages.setLanguageText(new Label(interferenceLog, SWT.WRAP), "Dasu.view.log.explanation");
		interferenceTable = new Table(interferenceLog, SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION | SWT.VIRTUAL);
		String[] colHeaders = {"Dasu.view.log.column.time", "Dasu.view.log.column.type","Dasu.view.log.column.status","Dasu.view.log.column.progress"};
		int[] sizes = {200,300,100,300};
		int[] aligns = { SWT.LEFT, SWT.LEFT, SWT.LEFT, SWT.LEFT};
	    for (int i = 0; i < colHeaders.length; i++) {
	      TableColumn tc = new TableColumn(interferenceTable, aligns[i]);
	      tc.setText(colHeaders[i]);
	      tc.setWidth(sizes[i]);
	      Messages.setLanguageText(tc, colHeaders[i]);
	    }
	    interferenceTable.setHeaderVisible(true);
	    gridData = new GridData(GridData.FILL_BOTH);
	    gridData.heightHint = interferenceTable.getHeaderHeight() * 6;
	    interferenceTable.setLayoutData(gridData);
	    interferenceTable.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub
			}
			public void widgetSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub
				if(interferenceTable.getSelectionCount()==1){
					updateDescriptionBox();
				}
			}
		});
		// </interferenceTable>
	    
	    // <Interference details box>
		gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);		
		descriptionPanel = new Composite(panel, SWT.NONE);
		descriptionPanel.setLayout(new GridLayout());
	    // </Interference description box>		

		//addDebugEntries(); //add test entries for debugging		
		//refresh();
		
		// set size of panels
		panel.setSize(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT, true));
		scrollPanel.setContent(panel);		
		scrollPanel.setMinHeight(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).y);
		scrollPanel.setMinWidth(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x);
		
		disable();
		refresh();

	}
    
	SelectionAdapter dotTorrentTestAdapter = new SelectionAdapter() {
        public void widgetSelected( SelectionEvent event ) {
        	AMCoordinatorV2.sendToLog(".torrent download blocking scheduled by user");
        	AMCoordinatorV2.assertToEngine(SourceType.RE, 
        			new FactFireAction("dotTorrent", "launchDotTorrentTest"));
            dotTorrentTestButton.setEnabled(false);
            dotTorrentTestTimer = System.currentTimeMillis();
        }
    };
	
    SelectionAdapter upnpTestAdapter = new SelectionAdapter() {
        public void widgetSelected( SelectionEvent event ) {
        	AMCoordinatorV2.sendToLog("Port blocking test scheduled by user");
        	AMCoordinatorV2.assertToEngine(SourceType.RE, 
        			new FactFireAction("portBlocking", "launchPortBlockingTest"));
        	upnpTestButton.setEnabled(false);
        	upnpTestTimer = System.currentTimeMillis();
        }
    };
    
    SelectionAdapter changeIspAdapter = new SelectionAdapter() {
        public void widgetSelected( SelectionEvent event ) {
        	changeIsp();
        }
    };
    private void changeIsp(){
    	ispShell = new Shell(display, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL); 
    	ispShell.setText( "Set ISP" ); 
    	ispShell.setSize(450,200);
    	GridLayout gridLayout = new GridLayout();
    	GridData gridData = new GridData();
    	gridLayout.numColumns = 1;
    	ispShell.setLayout(gridLayout);
    	Label selectLabel = new Label(ispShell, SWT.NONE);
    	selectLabel.setLayoutData(new GridData());
    	selectLabel.setText("Select ISP from the list or enter a new one.  ");
    	ispCombo = new Combo(ispShell, SWT.NONE);
    	if(reportedISP != null){
    		ispCombo.add(reportedISP);
    	}else {
    		// do isp name look up?	
    	}
    	
    	ispCombo.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
    	ispText = new Text(ispShell, SWT.BORDER | SWT.WRAP);
    	
		gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
	   
		ispText.setLayoutData(gridData);
		ispText.setTextLimit(100);
    	
    	Composite optionsButtonArea = new Composite(ispShell, SWT.NONE);
        gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = SWT.FILL;
		gridData.verticalAlignment = SWT.FILL;
		optionsButtonArea.setLayoutData(gridData);
		gridLayout = new GridLayout();
    	gridLayout.numColumns = 2;
    	optionsButtonArea.setLayout(gridLayout);
    	optionsButtonArea.setData(new GridData(GridData.FILL_HORIZONTAL));
    	
		Button submit = new Button(optionsButtonArea, 0);
        gridData = new GridData();
        submit.setLayoutData( gridData );
        submit.setText( "Select" );
        submit.setToolTipText( "Use selected or entered ISP name" );
        submit.addSelectionListener(new SelectionAdapter(){
        	public void widgetSelected( SelectionEvent event ) {
        		
        		if(ispText.getText().length() > 0){
        			ispName = ispText.getText();
        			
        		} else {
        			ispName = reportedISP;
        		}
        		ispLabel.setText("Current ISP: " + ispName);
    			String prefix = NetworkAdminASNLookupImpl.getInstance().getASNInfo().getBGPPrefix();
    			persistentDataManager.setIspInfo(new Pair(ispName, Util.currentGMTTime()));
    			Statistics.getInstance().commitIspInfo(prefix,ispName);
        		ispShell.dispose();
            }
        	
        });
        
        Button cancel = new Button(optionsButtonArea, 0);
        gridData = new GridData();
        cancel.setLayoutData( gridData );
        cancel.setText( "Cancel" );
        cancel.setToolTipText( "Cancel submission and close window" );
        cancel.addSelectionListener(new SelectionAdapter(){
        	public void widgetSelected( SelectionEvent event ) {
        		ispShell.dispose();
            }
        	
        });
    	
    	ispShell.open();
    	ispShell.setFocus();
    	ispShell.layout();
    }
    
    SelectionAdapter addCommentAdapter = new SelectionAdapter() {
        public void widgetSelected( SelectionEvent event ) {
        	// open a window for comments.
            newCommentEvent();
        }
    };
    
    private void newCommentEvent(){
    	//Dialog diaglop = new Dialog(shell);
    	commentShell = new Shell(display, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL); 
    	
    	commentShell.setSize(700,500);
    	commentShell.setText( "Submit comments" ); 
    	
    	GridLayout gridLayout = new GridLayout();
    	gridLayout.numColumns = 1;
    	commentShell.setLayout(gridLayout);
    			
		Label boxLabel = new Label(commentShell, SWT.WRAP);
		boxLabel.setText("Select a torrent (or all) that you suspect is experiencing interference and describe " +
				"what you believe is happening (comments are cut off at 500 characters).\nNOTE: We do not record the names of any torrent files.  " +
				"We assign a randomly generated ID to each torrent that is used to identify interference.");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 1;
		boxLabel.setLayoutData(gridData);
		
		
		Label torrentsLabel = new Label(commentShell, SWT.RIGHT);
		torrentsLabel.setText("Select torrent(s) (use control to select multiple torrents");
		
		torrentsLabel.setLayoutData(new GridData());    
        
		
		torrentComboList = new List(commentShell, (SWT.MULTI | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL));
		
		for (Download d : Main.getPluginInterface().getDownloadManager().getDownloads()) {
			torrentComboList.add(d.getName());
		}
		torrentComboList.add("All torrents");
		torrentComboList.add("All active torrents");
		torrentComboList.add("All seeding torrents");
		torrentComboList.add("All downloading torrents");
		
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.widthHint = commentShell.getSize().y;
		gridData.heightHint = torrentComboList.getItemHeight()*3;
		
		torrentComboList.setLayoutData(gridData);
		
		Label commentLabel = new Label(commentShell, SWT.RIGHT);
		commentLabel.setText("Enter Description");
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		commentLabel.setLayoutData(gridData);    
		
		commentText = new Text(commentShell, SWT.BORDER | SWT.WRAP | SWT.MULTI);
		
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
	   
		commentText.setLayoutData(gridData);
		commentText.setTextLimit(500);
		
		Composite commentButtonArea = new Composite(commentShell, SWT.NONE);
        gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = SWT.FILL;
		gridData.verticalAlignment = SWT.FILL;
		commentButtonArea.setLayoutData(gridData);
		gridLayout = new GridLayout();
    	gridLayout.numColumns = 2;
		commentButtonArea.setLayout(gridLayout);
		commentButtonArea.setData(new GridData(GridData.FILL_HORIZONTAL));

		Button submit = new Button(commentButtonArea, 0);
        gridData = new GridData();
        submit.setLayoutData( gridData );
        submit.setText( "Submit" );
        submit.setToolTipText( "Submit report of undetected interference" );
        submit.addSelectionListener(submitCommentAdapter);
		
        Button cancel = new Button(commentButtonArea, 0);
        gridData = new GridData();
        cancel.setLayoutData( gridData );
        cancel.setText( "Cancel" );
        cancel.setToolTipText( "Cancel submission and close window" );
        cancel.addSelectionListener(cancelCommentAdapter);
        
        commentShell.open();
        commentShell.setFocus();
    	commentShell.layout();
    }
    
    
    
    SelectionAdapter submitCommentAdapter = new SelectionAdapter() {
        public void widgetSelected( SelectionEvent event ) {
        	HashSet<String> torrentSet = new HashSet<String>();
        	interferenceReportTorrentIds = "";        	
        	
        	if(torrentComboList.getSelectionCount() !=0){
        		for(int i=0; i<torrentComboList.getSelectionCount(); i++){
            		torrentSet.add(torrentComboList.getSelection()[i]);
            	}
        		
        		for (Download d : Main.getPluginInterface().getDownloadManager().getDownloads()) {
        			if(torrentSet.contains("All torrents")){
        				interferenceReportTorrentIds += Main.getId(d) + ", ";
        			}else if(torrentSet.contains("All downloading torrents") && d.getState() == d.ST_DOWNLOADING){
        				interferenceReportTorrentIds += Main.getId(d) + ", ";
        			}else if(torrentSet.contains("All seeding torrents") && d.getState() == d.ST_SEEDING){
        				interferenceReportTorrentIds += Main.getId(d) + ", ";
        			}else if(torrentSet.contains(d.getName())){
        				interferenceReportTorrentIds += Main.getId(d) + ", ";
        			}
        		}
        		interferenceReportComments = commentText.getText();
        		//System.out.println("!!!!" + interferenceReportTorrentIds + " " + interferenceReportComments);
        		Statistics.getInstance().commitUserComments(interferenceReportTorrentIds, interferenceReportComments);
        	
        	}
        	
        	commentShell.dispose();
        }
    };
    
    
    
    public String getInterferenceReportTorrentIds() {
		return interferenceReportTorrentIds;
	}

	public String getInterferenceReportComments() {
		return interferenceReportComments;
	}

	SelectionAdapter cancelCommentAdapter = new SelectionAdapter() {
        public void widgetSelected( SelectionEvent event ) {
        	commentShell.dispose();
        }
    };
	
	private void updateDescriptionBox(){
		HVLogEntry entry = interferenceNotices.get(interferenceTable.getSelectionIndex());
		
		// create new descriptionPanel
		descriptionPanel.dispose();
		descriptionPanel = new Composite(panel, SWT.BORDER);
		
		GridLayout gridLayout = new GridLayout();
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);		
		
		gridLayout.numColumns=2;
		descriptionPanel.setLayout(gridLayout);
		descriptionPanel.setLayoutData(gridData);
		
		ArrayList<Object> objectList = entry.getDescriptionObjects();
		Iterator<Object> itr = objectList.iterator();
		int count = 0;
		while(itr.hasNext()){
			Object object = itr.next();
			if(object.getClass() == ArrayList.class){
				ArrayList<String> strings = (ArrayList<String>) object;
				Composite textRegion = new Composite(descriptionPanel, SWT.NONE);
				textRegion.setLayout(new GridLayout());
				gridData = new GridData(GridData.FILL_VERTICAL);
				gridData.widthHint = 300;
				
				textRegion.setLayoutData(gridData);
				
				Iterator<String> strItr = strings.iterator();
				while(strItr.hasNext()){
					Label boxLabel = new Label(textRegion, SWT.WRAP);
					
					boxLabel.setText((String) strItr.next());
					gridData = new GridData(GridData.FILL_HORIZONTAL);
					gridData.widthHint = 200;  // needed for wrap
					boxLabel.setLayoutData(gridData);
				}
			}
			if(object.getClass() == HVRow.class){
				Composite textRegion = new Composite(descriptionPanel, SWT.NONE);
				gridData = new GridData(GridData.FILL_HORIZONTAL);
				gridData.horizontalSpan = 2;
				textRegion.setLayoutData(gridData);
				textRegion.setLayout(new GridLayout());
				
				HVRow row = (HVRow) object;
				Label boxLabel = new Label(textRegion, SWT.WRAP);
				if(row.getObject().getClass() == String.class){
					boxLabel.setText((String)row.getObject());	
				}
				
				gridData = new GridData(GridData.FILL_HORIZONTAL);
				gridData.horizontalSpan = 2;
				boxLabel.setLayoutData(gridData);
				gridData = new GridData(GridData.FILL_VERTICAL);
				gridData.widthHint = panel.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x;
			}
			if(object.getClass() == String.class){
				Composite textRegion = new Composite(descriptionPanel, SWT.NONE);
				textRegion.setLayout(new GridLayout());
				gridData = new GridData(GridData.FILL_VERTICAL);
				gridData.widthHint = panel.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x/2;
				textRegion.setLayoutData(gridData);
				
				Label boxLabel = new Label(textRegion, SWT.WRAP);
				
				boxLabel.setText((String) object);
				gridData = new GridData(GridData.FILL_HORIZONTAL);
				gridData.widthHint = 200;  // needed for wrap
				boxLabel.setLayoutData(gridData);
			}
			else if(object.getClass() == HVPlot.class){
				
				HVPlot plotData = (HVPlot) object;
				
				if (!plotData.getTitle().equals("empty")) {
				
					Composite graphRegion = new Composite(descriptionPanel, SWT.NONE);
					graphRegion.setLayout(new GridLayout());
					graphRegion.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL));
					
					
					Label boxLabel = new Label(graphRegion, SWT.WRAP);
					boxLabel.setText((String) plotData.getTitle());
					gridData = new GridData(GridData.FILL_HORIZONTAL);
					gridData.widthHint = 200;  // needed for wrap
					boxLabel.setLayoutData(gridData);
					
					Canvas graph = new Canvas(graphRegion,SWT.BORDER);
					graph.setBackground(Colors.white);
					graph.setLayout(new GridLayout());
					gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);
					graph.setLayoutData(gridData);
			        graph.addPaintListener(new PointExamplePaintListener(plotData));
			        
				    graph.layout();
				    count++;
				}
			    
				/**MARIO**/
				
//			    HVPlot plotData = (HVPlot) object;
//			    if (plotData.getPoints() != null ) {
//				
//				    Canvas downSpeedCanvas;
//				    SpeedGraphic downSpeedGraphic;
//					
//				    Group gDownSpeed = new Group(panel,SWT.NULL);
//				    Messages.setLanguageText(gDownSpeed, "Download Speeds");
//				    gridData = new GridData(GridData.FILL_BOTH);
//				    gDownSpeed.setLayoutData(gridData);    
//				    gDownSpeed.setLayout(new GridLayout());
//				    
//				    downSpeedCanvas = new Canvas(gDownSpeed,SWT.NO_BACKGROUND);
//				    downSpeedCanvas.setLayoutData(gridData);
//				    downSpeedGraphic = SpeedGraphic.getInstance();
//				    downSpeedGraphic.initialize(downSpeedCanvas);
//				    
//			        int[] pings = new int[plotData.getPoints().size()];
//			        int i=0;
//				    for ( Pair<Long, Float> v : plotData.getPoints()) {
//				          pings[i++] = v.getValue().intValue();
//				    }				    
//			    	 downSpeedGraphic.addIntsValue(pings);
//			    	 downSpeedGraphic.refresh();
//				    				    
//				    for (i=0; i<400; i++) {
//				    	 downSpeedGraphic.addIntsValue(new int[]{i});
//				    	 downSpeedGraphic.refresh();			    	
//				    }
//				    
//				    count++;
//			    }
			    
			}
		}
		
		
		descriptionPanel.setSize(descriptionPanel.computeSize(SWT.DEFAULT, SWT.DEFAULT, true));
		scrollPanel.setMinHeight(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).y+count*200);
		panel.layout();
	}
	
	private class PointExamplePaintListener implements PaintListener {
		HVPlot plot;
		public PointExamplePaintListener(HVPlot plot) {
			this.plot = plot;
		}

		public void paintControl(PaintEvent e) {
			Canvas canvas = (Canvas) e.widget;
			GC gc = e.gc;
			Rectangle bounds = canvas.getClientArea();
			ArrayList<Pair<Long, Float>> points = plot.getPoints();
			double max_y = 0;
			
			//double min_y = Double.MAX_VALUE;	// Y axis starts at next nice label
			double min_y = 0;	// Y axis always starts at 0
			long max_x = 0;
			long min_x = Long.MAX_VALUE;
			for(int i=0;i<points.size();i++){
				max_x = Math.max(max_x, (points.get(i).getKey()));
				min_x = Math.min(min_x, (points.get(i).getKey()));
				max_y = Math.max(max_y, (points.get(i).getValue()));
				min_y = Math.min(min_y, (points.get(i).getValue()));
			}
			
			int numTick = 5;
			int nfrac;
			double d;
			double graphmin, graphmax;
			double range;
			range = findNiceNum(max_y-min_y, false);
			d = findNiceNum(range/(numTick-1),true);
			graphmin = Math.floor(min_y/d)*d;
			graphmax = Math.ceil(max_y/d)*d;
			max_y = graphmax;
			min_y = graphmin;
			nfrac = (int) Math.max(-Math.floor(Math.log10(d)), 0);
			int maxStrLength = 0;
			for(double i = graphmin; i <= graphmax + .5*d;i=i+d){
				String formatString = "%." + nfrac + "f" ;
				String str = String.format(formatString, i);
				maxStrLength = Math.max(str.length(),maxStrLength);
			}
			
			final int	PAD_LEFT = maxStrLength*8+10;
			final int	PAD_TOP	= 30;
			final int	PAD_BOTTOM = 40;
			final int	PAD_RIGHT = 30;
			int usable_width 	= bounds.width - PAD_LEFT - PAD_RIGHT;
			int usable_height	= bounds.height - PAD_TOP - PAD_BOTTOM;
			// x axis
			int x_axis_left_x =  PAD_LEFT;
			int x_axis_left_y = usable_height + PAD_TOP;
			int x_axis_right_x = PAD_LEFT + usable_width;
			int x_axis_right_y = x_axis_left_y;

			gc.drawLine( x_axis_left_x, x_axis_left_y, x_axis_right_x, x_axis_right_y );

			// y axis
			int y_axis_bottom_x = PAD_LEFT;
			int y_axis_bottom_y = usable_height + PAD_TOP;
			int y_axis_top_x 	= PAD_LEFT;
			int y_axis_top_y 	= PAD_TOP;
			gc.drawLine( y_axis_bottom_x, y_axis_bottom_y, y_axis_top_x, y_axis_top_y );
			
			
			double x_ratio = (double)usable_width/(max_x-min_x);
			double y_ratio = (double)usable_height/(graphmax-graphmin);
			
			
			for(double i = graphmin; i <= graphmax + .5*d;i=i+d){
				int y1 = (PAD_TOP+usable_height-(int)(y_ratio*(i-min_y)));
				String formatString = "%." + nfrac + "f" ;
				String str = String.format(formatString, i);
				int length = str.length();
				gc.drawText(str, (maxStrLength-length)*8+3, (int)y1-10);
				gc.drawLine(x_axis_left_x-3, y1, x_axis_left_x+3, y1);
			}
			
			if(plot.isShowHighlight()){
				gc.setBackground(Colors.grey);
				int x1 = PAD_LEFT+(int)(x_ratio*(plot.getX1Highlight()-min_x));
				int x2 = PAD_LEFT+(int)(x_ratio*(plot.getX2Highlight()-min_x));
				gc.fillRectangle(x1, PAD_TOP, x2-x1, usable_height);
				gc.setBackground(Colors.white);
			}

			if(plot.getType()==HVPlot.LINE_GRAPH){
				gc.setForeground(Colors.blue);
				for (int i=0;i<points.size()-1;i++){
					int x1 = PAD_LEFT+(int)(x_ratio*(points.get(i).getKey()-min_x));
					int x2 = PAD_LEFT+(int)(x_ratio*(points.get(i+1).getKey()-min_x));
					int y1 = PAD_TOP+usable_height-(int)(y_ratio*(points.get(i).getValue()-min_y));
					int y2 = PAD_TOP+usable_height-(int)(y_ratio*(points.get(i+1).getValue()-min_y));
					gc.drawLine(x1, y1, x2, y2);
				}
			} else if(plot.getType()==HVPlot.SCATTER_GRAPH){
				gc.setForeground(Colors.red);
				for (int i=0;i<points.size();i++){
					int x1 = PAD_LEFT+(int)(x_ratio*(points.get(i).getKey()-min_x));
					int y1 = PAD_TOP+usable_height-(int)(y_ratio*(points.get(i).getValue()-min_y));		
					gc.drawOval(x1-2, y1-2, 4, 4);
				}
			}
			
			SimpleDateFormat format = null;
			if(plot.getXLabelFormat() == HVPlot.FORMAT_TIME)
				 format = new SimpleDateFormat("hh:mm:ss");
			else if(plot.getXLabelFormat() == HVPlot.FORMAT_DATE)
				format = new SimpleDateFormat("MM/dd/yy");
			else if(plot.getXLabelFormat() == HVPlot.FORMAT_MONTH)
				format = new SimpleDateFormat("MMM");
			else if(plot.getXLabelFormat() == HVPlot.FORMAT_DAY_WEEK)
				format = new SimpleDateFormat("EEE");
			
			gc.setForeground(Colors.black);
			
			Long numTics = max_x/((max_x - min_x)/4);
			Integer inc = points.size()/numTics.intValue();

			for(long i = min_x; i <= max_x; i= i + (max_x - min_x)/4){
				int x1 = (PAD_LEFT+(int)(x_ratio*(i-min_x)));
				//String formatString = "%." + nfrac + "f" ;
				
				String str = String.valueOf(i);
				if ((plot.getXLabelFormat() != HVPlot.FORMAT_NUMBER) && 
					(i > 365)) {
					Date current = new Date(i);
					str = format.format(current);
				}
				else if ( ((plot.getXLabelFormat() == HVPlot.FORMAT_DAY_WEEK) || 
						(plot.getXLabelFormat() == HVPlot.FORMAT_MONTH)) &&
						(i <= 365)) {

					Calendar current = Calendar.getInstance();
					current.set(Calendar.DAY_OF_WEEK, (int) i%7);
					
					current.set(Calendar.MONTH, (int) i%12-1);
					
					str = format.format(new Date(current.getTimeInMillis()));
				}

				int shift = str.length()/2+4;
				gc.drawText(str, x1-shift, usable_height+PAD_TOP+5);
				gc.drawLine(x1, usable_height+PAD_TOP+2, x1, usable_height+PAD_TOP-2);
			}
		
			/*xlabel, ylabel*/
			gc.drawText(plot.getxLabel(), (x_axis_right_x + x_axis_left_x)/2, y_axis_bottom_y+5*PAD_BOTTOM/8);
			gc.drawText(plot.getyLabel(), x_axis_left_x - 3*PAD_LEFT/4, y_axis_top_y - 7*PAD_TOP/8);
			
		}
	}

	public int findNiceNum(double x, boolean roundDown){
		int exp;
		double f; 
		double nf;
		exp = (int) Math.floor(Math.log10(x));
		f = x/Math.pow(10, exp);
		if(roundDown){
			if(f < 1.5){
				nf = 1;
			} else if(f < 3){
				nf = 2;
			} else if(f < 7){
				nf = 5;
			} else {
				nf = 10;
			}
		} else {
			if(f <= 1){
				nf = 1;
			} else if(f<=2){
				nf = 2;
			} else if(f<=5){
				nf = 5;
			} else {
				nf = 10;
			}
		}
		
		return (int) (nf*Math.pow(10, exp));
	}
	
	public void addNewInterferenceEntry(HVLogEntry entry){
		interferenceNotices.add(entry);
	}
	
	public ArrayList<HVLogEntry> getInterferenceEvents(){
		return interferenceNotices;
	}
	
	public void setInterferenceEvents(ArrayList<HVLogEntry> entries){
		interferenceNotices = entries;
	}
	
	public void updateInterferenceEvent(int index, HVLogEntry entry){
		interferenceNotices.set(index, entry);
	}
	
	public void updateTable(){

	}
	
	public String getFullTitle() {
		return MessageText.getString("Dasu.fullname");
	}

	public String getData() {
		return "DHTView.title.full";
	}

	public boolean eventOccurred(UISWTViewEvent event) {
		switch (event.getType()) {
			case UISWTViewEvent.TYPE_CREATE:
				if (isCreated){
					return false;
				}
				isCreated  = true;
				break;

			case UISWTViewEvent.TYPE_DESTROY:
				delete();
				isCreated = false;
				break;

			case UISWTViewEvent.TYPE_INITIALIZE:
				initialize((Composite)event.getData());
				break;

			case UISWTViewEvent.TYPE_LANGUAGEUPDATE:
				//updateLanguage();
				break;

			case UISWTViewEvent.TYPE_REFRESH:
				refresh();
				break;
		}
		return true;
	}

	public void delete() {
		Utils.disposeComposite(panel);
	}

	public void refresh() {
		
		if ((interferenceTable == null) || (interferenceNotices == null))
			return;
		
		synchronized(this){
			interferenceTable.setItemCount(interferenceNotices.size());
		}
		
		TableItem[] test = interferenceTable.getItems();
		for(int i=0; i < interferenceNotices.size();i++){
			test[i].setText(0, DateFormat.getDateTimeInstance().format(interferenceNotices.get(i).getTime()));
			test[i].setText(1, interferenceNotices.get(i).getType());
			if(interferenceNotices.get(i).getStatus() != null){
				test[i].setText(2, interferenceNotices.get(i).getStatus());
			} else {
				test[i].setText(2, "unknown");
			}
			
			if(interferenceNotices.get(i).getProgress() != null){
				test[i].setText(3, interferenceNotices.get(i).getProgress());
			}
			
			DateFormat.getDateTimeInstance().format(interferenceNotices.get(i).getTime());

		}
		
		if (!DasuConfiguration.getInstance().isRecordStats())
			disable();
		else 
			enable();
		
		refreshIspInfo();
		
	}
	
	public void addDebugEntries(){
		HVLogEntry testentry = new HVLogEntry();
		testentry.setTime(System.currentTimeMillis()-1000000);
		testentry.setType("Interference event 1");

		ArrayList<Object> list1 = new ArrayList<Object>();
		ArrayList<String> stringList = new ArrayList<String>();
		stringList.add("Description: Drop in download speed");
		stringList.add("Time: "+DateFormat.getDateTimeInstance().format(testentry.getTime()));

		list1.add(stringList);
		
		HVPlot temp = new HVPlot();
		
		ArrayList<Pair<Long, Float>> points = new ArrayList<Pair<Long, Float>>();
		int x = 0;
		for(int i=0;i<100;i++){
			Pair<Long,Float> mypair = new Pair<Long,Float>((long)x, (float) i);
			points.add(mypair);
			x++;
		}
		for(int i=100;i>0;i=i-4){
			Pair<Long,Float> mypair = new Pair<Long,Float>((long)x, (float) i);
			points.add(mypair);
			x++;
			
		}
		temp.setPoints(points);
		temp.setTitle("Download Speeds Dropping");
		temp.setXLabel("Time");
		temp.setType(HVPlot.LINE_GRAPH);
		list1.add(temp);
		
		testentry.setDescriptionObjects(list1);
		interferenceNotices.add(testentry);
		
		/**********************************************/
		
		testentry = new HVLogEntry();
		testentry.setTime(System.currentTimeMillis());
		testentry.setType("Interference event 2");
		list1 = new ArrayList<Object>();
		stringList = new ArrayList<String>();

		stringList.add("Description: Drop in download speed");
		stringList.add("Time: "+DateFormat.getDateTimeInstance().format(testentry.getTime()));
		list1.add(stringList);
		
		temp = new HVPlot();
		Pair<Long,Float> mypair;
		points = new ArrayList<Pair<Long, Float>>();
		mypair = new Pair<Long,Float>(1269315895100l, (float) 8000.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269315926332l, (float) 8088.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269315957466l, (float) 8173.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269315988874l, (float) 7967.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316020036l, (float) 8022.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316051210l, (float) 9149.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316082364l, (float) 7611.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316113518l, (float) 9502.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316144974l, (float) 9518.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316176439l, (float) 7583.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316207615l, (float) 7478.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316239048l, (float) 7409.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316270240l, (float) 7721.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316301453l, (float) 8000.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316332851l, (float) 7906.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316364059l, (float) 8000.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316395485l, (float) 7100.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316426675l, (float) 8000.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316457843l, (float) 7627.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316489255l, (float) 6469.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316520682l, (float) 7545.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316551864l, (float) 8000.0);
		points.add(mypair);
		mypair = new Pair<Long,Float>(1269316583321l, (float) 7349.0);
		points.add(mypair);
		temp.setShowHighlight(true);
		temp.HighlightSection(1269316332851l, 1269316489255l);
		temp.setPoints(points);	
		temp.setTitle("Drop in download rate");
		temp.setXLabel("Time");
		temp.setType(HVPlot.LINE_GRAPH);
		list1.add(temp);
	
		HVRow row = new HVRow("hey hey heyasdffffffffffffffffffffffffffffffffffffffffffffffffffawefkaldfjlaskdjfllllkasdflaksjdfalsdjflasdjfla asdfl as r;flaj f;laks flasj flaskd lfkja sldf alsk flaskj dflajk sdlfj asldjf alsjd flasj dflasjk dflja sdlfja slfj asldjkf alsdj flaskdj flasdjk fskjdfljasdflkjasf");
		list1.add(row);
		list1.add(stringList);
		list1.add(temp);
		testentry.setDescriptionObjects(list1);
		interferenceNotices.add(testentry);
	}
}


