/**
 * Dasu Project
 *
 * File:         DasuView.java
 * RCS:          $Id: InterferenceEventUI.java,v 1.4 2010/10/22 19:51:35 mas939 Exp $
 * Description:  DasuView class (see below)
 * Author:       Zachary Bischof
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 26, 2010 at 12:21:13 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.ui
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.stats.DownloadStatsDynamic;
import edu.northwestern.dasu.stats.DynamicLocalPeerStat;
import edu.northwestern.dasu.stats.RSTStat;
import edu.northwestern.dasu.stats.NetstatSystemStats;
import edu.northwestern.dasu.util.Pair;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The DasuView class ...
 */

public class InterferenceEventUI implements Serializable {

	private int errors = 0;
	private HVLogEntry interferenceEntry = null;
	private ArrayList<Object> msgList = null;
	
	private static boolean DEBUG = true;
	
	public InterferenceEventUI(String interfEventDesc, long did) {
	
		interferenceEntry = new HVLogEntry();
		new ArrayList<String>();
		msgList = new ArrayList<Object>();
		interferenceEntry.setDescriptionObjects(msgList);
		
		long time = System.currentTimeMillis();

		if (DasuManager.didFactMap.containsKey(did)) {
			interfEventDesc = interfEventDesc.concat(", (torrent: " + DasuManager.didFactMap.get(did).getDownloadName() + ")");
		}
		else {
			errors++;
			if (DEBUG)
				System.out.println("InterferenceEventUI: " + did + " does not exist! size: " + DasuManager.didFactMap.size());
		}
		
		interferenceEntry.setTime(time);
		interferenceEntry.setType(interfEventDesc);
		
	}
	
	public InterferenceEventUI(String interfEventDesc) {
		
		interferenceEntry = new HVLogEntry();
		msgList = new ArrayList<Object>();
		interferenceEntry.setDescriptionObjects(msgList);
		interferenceEntry.setTime(System.currentTimeMillis());
		interferenceEntry.setType(interfEventDesc);
		
	}
	
	public HVLogEntry getHVLogEntry() {
		return this.interferenceEntry;
	}
	
	public void addDescriptionLine(String line) {
	
		ArrayList<String> description = new ArrayList<String>(); 
		description.add(line);
		msgList.add(description);

	}
	
	public void addDescriptionLine(String[] list) {

		msgList.add(new ArrayList<String>(Arrays.asList(list)) );
		
	}
	
	public void addFullScreenDescriptionLine(String line) {
		HVRow row = new HVRow(line);
		msgList.add(row);
	}
	
	public void addPlot(String type, String field, String title, String xLabel, String yLabel) {
		
		HVPlot plot = drawPlot(type, field, title, xLabel, yLabel); 
		if (plot != null)
			msgList.add(plot);
		else
			errors++;
		

	}
	
	public void addPlot(String type, Long did, String field, String title, String xLabel, String yLabel) {

		HVPlot plot = drawPlot(type, did, field, title, xLabel, yLabel); 
		if (plot != null)
			msgList.add(plot);
		else {
			if (DEBUG) 
				System.out.println("addPlot: " + did + " does not exist!");
			errors++;
		}
	}

	public void addPlot(String title, Collection<Float> yValues, String xLabel, String yLabel) {
		addPlot(title, null, yValues, xLabel, yLabel, -1, -1);
	}
	
	public void addPlot(String title, Float[] yValues, String xLabel, String yLabel) {
		addPlot(title, Arrays.asList(yValues), xLabel, yLabel);
	}

	public void addPlot(String title, Collection<Long> xValues, Collection<Float> yValues, String xLabel, String yLabel) {
		addPlot(title, xValues, yValues, xLabel, yLabel, -1, -1);
	}	
	
	public void addPlot(String title, Collection<Long> xValues, Collection<Float> yValues, 
			String xLabel, String yLabel, int plotType, int xLabelFormat) {
		
		ArrayList<Long> xAxis = new ArrayList<Long>();
		
		if (xValues != null) {
			
			Iterator<Long> iter = xValues.iterator();
			long first = iter.next();
			long last = first;
			while (iter.hasNext())
				last = iter.next();

			if (last == first) {			
				
				for (long i=1; i<= yValues.size(); i++)
					xAxis.add(i);
			}
			else {
				xAxis = (ArrayList<Long>) xValues;
			}
		}
		else {
			for (long i=1; i<= yValues.size(); i++)
				xAxis.add(i);
		}
		
		HVPlot plot = createPlot(title, xAxis, yValues, xLabel, yLabel, plotType, xLabelFormat);
		
		if (plot != null)
			msgList.add(plot);
		else {
			if (DEBUG) 
				System.out.println("addPlot: error with supplied values!");
			errors++;
		}
		
		
	}

	public void drawUI() {
		
		if (errors == 0) {
			interferenceEntry.setDescriptionObjects(msgList);
			DasuView.getInstance().addNewInterferenceEntry(interferenceEntry);
		}
		else
			System.out.println("InterferenceDrawUI: There has been an error!!");

	}
	
	private HVPlot drawPlot(String type, Long did, String field, String title, String xLabel, String yLabel) {

		ArrayList<Pair<Long, Float>> dataValues = new ArrayList<Pair<Long, Float>>();

		try {

			if (type.equals("DownloadStatsDynamic")) {

				if (DasuManager.didFactMap.containsKey(did)) {
					
					Iterator iter = DasuManager.didFactMap.get(did).entryLog.iterator();
	
					if (DEBUG) 
						System.out.println("didFact plot");
					
					while (iter.hasNext()) {
						
						DownloadStatsDynamic entry = (DownloadStatsDynamic) iter.next();
					    Long timeRecorded = entry.timeRecorded;
					    float val = new Float ((Integer)entry.getClass().getField(field).get(entry));
						dataValues.add(new Pair<Long, Float>(timeRecorded + Util.dasuTimeOffset, val));
						
						if (DEBUG)
							System.out.println(timeRecorded + " " + val);
					}
									
				}
			}
			else if (type.equals("RSTStat")) {
				
				if (DasuManager.rstFactMap.containsKey(did)) {
					
					Iterator iter = DasuManager.rstFactMap.get(did).entryLog.iterator();
	
					if (DEBUG) 
						System.out.println("rstFact plot");
					
					while (iter.hasNext()) {
						
						RSTStat entry = (RSTStat) iter.next();
					    Long timeRecorded = entry.timeRecorded;
					    float val = new Float ((Integer)entry.getClass().getField(field).get(entry));
						dataValues.add(new Pair<Long, Float>(timeRecorded + Util.dasuTimeOffset, val));

						
						if (DEBUG)
							System.out.println(timeRecorded + " " + val);
						
					}
				}
				
			}
		
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (dataValues.size() > 0) {
			HVPlot plot = new HVPlot(dataValues, title, xLabel, yLabel, -1, -1);
			System.out.println(dataValues);
			return plot;
		}
		
		
		return null;

		
	}
	
	public HVPlot createPlot(String title, Collection<Long> xValues, Collection<Float> yValues, 
			String xLabel, String yLabel, int plotType, int xLabelFormat) {
		
		ArrayList<Pair<Long, Float>> dataValues = new ArrayList<Pair<Long, Float>>();
		
		if ((xValues != null) && (yValues != null) && (xValues.size() == yValues.size())) {
			
			Iterator<Long> xIter = xValues.iterator();
			Iterator<Float> yIter = yValues.iterator();
			while (xIter.hasNext()) {
				dataValues.add(new Pair<Long, Float>(xIter.next(), yIter.next()));
			}
			
			if (dataValues.size() > 0) {
				HVPlot plot = new HVPlot(dataValues, title, xLabel, yLabel, plotType, xLabelFormat);
				return plot;
			}
			
		}
		else {
			System.out.println("createPlot: Error, invalid array size...\n");
		}
		
		return new HVPlot(); 
		
 	}
	
	private HVPlot drawPlot(String type, String field, String title, String xLabel, String yLabel) {

		try {
			
			Iterator iter = null;
			ArrayList<Pair<Long, Float>> dataValues = new ArrayList<Pair<Long, Float>>();

			if (type.equals("RSTSystemStats")) {
				iter = DasuManager.systemFact.entryLog.iterator();
				
				if (DEBUG)
					System.out.println("RSTSystemStats PLOT");
				
				while (iter.hasNext()) {
					
					NetstatSystemStats entry = (NetstatSystemStats) iter.next();
				    Long timeRecorded = entry.timeRecorded;
				    float val = new Float( (Integer) entry.getClass().getField(field).get(entry) );
					dataValues.add(new Pair<Long, Float>(timeRecorded + Util.dasuTimeOffset, val));
					
					if (DEBUG)
						System.out.println(timeRecorded + " " + val);
					
				}
			}
			
			else if (type.equals("DynamicLocalPeerStat")) {
				iter = DasuManager.localPeerFact.entryLog.iterator();
				
				while (iter.hasNext()) {
					
					DynamicLocalPeerStat entry = (DynamicLocalPeerStat) iter.next();
				    Long timeRecorded = entry.currentTime;
				    Float val = 0F;
				    if (entry.getClass().getField(field).get(entry) instanceof Integer)
				    //if (entry.getClass().getField(field).get(entry).getClass().toString().indexOf("Integer") != -1)
				    	val = ((Integer) entry.getClass().getField(field).get(entry)).floatValue();
				    else
				    	val = (Float) entry.getClass().getField(field).get(entry);
					
				    dataValues.add(new Pair<Long, Float>(timeRecorded +  Util.dasuTimeOffset, val));

				}
			}			
			
			if (dataValues.size() > 0) {
				HVPlot plot = new HVPlot(dataValues, title, xLabel, yLabel, -1, -1);
				return plot;
			}
			
			
			return new HVPlot();


		
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return new HVPlot();		
	}

	public void clearDescription() {
		this.interferenceEntry.getDescriptionObjects().clear();
	}
	
	public void setStatus(String status) {
		this.interferenceEntry.setStatus(status);
		setTime(System.currentTimeMillis());
	}
	
	public String getStatus() {
		return this.interferenceEntry.getStatus();
	}
	
	public void setTime(Long time) {
		this.interferenceEntry.setTime(time);
	}

	public Long getTime() {
		return this.interferenceEntry.getTime();
	}
	
	public void setProgress(String progress) {
		this.interferenceEntry.setProgress(progress);
		setTime(System.currentTimeMillis());
	}
	
	public String getProgress() {
		return this.interferenceEntry.getProgress();
	}

	
}

