/**
 * Dasu Project
 *
 * File:         DasuView.java
 * RCS:          $Id: UIManager.java,v 1.3 2010/05/31 22:20:05 mas939 Exp $
 * Description:  DasuView class (see below)
 * Author:       Zachary Bischof
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 26, 2010 at 12:21:13 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.ui
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The DasuView class ...
 */

public class UIManager implements Serializable {

	LinkedHashMap<String, InterferenceEventUI> events;

	Long lastUpdate = -1L;
	
	public UIManager() {
		//<String, InterferenceEventUI>
		events = new LinkedHashMap<String, InterferenceEventUI>();
		lastUpdate = System.currentTimeMillis();
	}
	
	public void drawUI() {
		
		ArrayList<HVLogEntry> array = new ArrayList<HVLogEntry>();
		Iterator<InterferenceEventUI> iter = events.values().iterator();
		while (iter.hasNext()) {
			InterferenceEventUI ie = (InterferenceEventUI) iter.next();
			array.add(ie.getHVLogEntry());
		}
		
		DasuView.getInstance().setInterferenceEvents(array);		
	}

	public InterferenceEventUI getEvent(String eventName) {
		if (!events.containsKey(eventName))
			events.put(eventName, new InterferenceEventUI(eventName));
			
		return (InterferenceEventUI) events.get(eventName);			
	}
	
	public InterferenceEventUI getEvent(String eventName, Long did) {
		if (!events.containsKey(eventName+did.toString()))
			events.put(eventName+did.toString(), new InterferenceEventUI(eventName, did));
			
		return (InterferenceEventUI) events.get(eventName+did.toString());			
	}

	public void setEvent(String name, InterferenceEventUI event) {
		events.put(name, event);
		drawUI();
	}
	
	public void setEvent(String eventName, Long did, InterferenceEventUI event) {
		events.put(eventName+did.toString(), event);
		drawUI();
	}

}

